package pa_model;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.swing.JOptionPane;

import cern.jet.random.Uniform;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.ui.RSApplication;
import repast.simphony.random.RandomHelper;

public class Util {
	public static final int DATA_FILES_HEADER_LINES = 2; // header lines to be ignored when reading input data files
	///////Multipliers constant in order to easily change simulation timestep 
	public static final double CONVERT_YEARS = 0.25; //multiplier to convert current simulation time to years 
	public static final double CONVERT_MONTHS = 3; //multiplier to convert current simulation time to months
	public static final double CONVERT_DAYS = 90; //multiplier to convert current simulation time to days
		
	//For PSA error check
	private static boolean psaError = false;
	private static String psaErrorMessage = "Not enough PSA data in: ";
	
	//Discount rates
	public static final double DISCOUNT_ANNUAL_RATE_COST = 0.035;
	public static final double DISCOUNT_ANNUAL_RATE_QALYS = 0.035;
	
	public static final double DISCOUNT_DAILY_RATE_COST = 1/Math.pow(1+DISCOUNT_ANNUAL_RATE_COST, 1/365);
	public static final double DISCOUNT_DAILY_RATE_QALYS = 1/Math.pow(1+DISCOUNT_ANNUAL_RATE_QALYS, 1/365);
		
	//public static final double DISCOUNT_DAILY_RATE_COST = Math.pow(1+DISCOUNT_ANNUAL_RATE_COST, 1/365) - 1;
	//public static final double DISCOUNT_DAILY_RATE_QALYS = Math.pow(1+DISCOUNT_ANNUAL_RATE_COST, 1/365) - 1;
	//Daily Increment
	public static final double DAILY_INCREMENT = 0.02;
	
	//fixed cost of active interventions TODO reset in PAContext for each run
	public static double totalIntrventionsFixedCost, totalHealthFixedCost, totalSportFixedCost, totalEducationFixedCost, totalTransportFixedCost,
	totalLocalAuthFixedCost, totalOtherFixedCost; 
	public static int interventionCount;
	
	//TODO threshold for considering a person active 
	public static final double PA_ACTIVE_THRESHOLD = (double) PAContext.p.getValue("activeThreshold"); // for counting active years
	
	// inactive if pa < 86 PA_MODERATELY_THRESHOLD
	public static final double PA_MODERATELY_THRESHOLD = 86; // min per week - moderately active if pa >=86 & pa < 426 (PA_VERY_THRESHOLD) 
	public static final double PA_VERY_THRESHOLD = 426; // min per week - very active if pa >=426
	
	//A minute change in moderate to vigorous physical activity (MVPA) was associated with utility change of 0.000027
	public static final double PA_UTILITY_CHANGE = 0.000027;
	
	public static File currentOutDirectory; 
	public static String dateAppend;
	
		public Util(){
			
			}
		//Simulation clock - advances every 3 months (repast tick); resets at start of each replication  
		public static class Clock{
			
			private double clockYears, clock3Months, clockMonths;
			
			public Clock(){
				resetClock();
			}
			
			@ScheduledMethod(start=0, interval=1, priority=ScheduleParameters.FIRST_PRIORITY)
			public void advanceClock(){
				
				this.setClock3Months(this.getClock3Months()+1);
				this.setClockYears(this.getClock3Months());
				this.setClockMonths(this.getClock3Months());
				Util u = new Util();
				if((this.getClock3Months()%4==0) && (this.getClockYears()<PAContext.clockYearsAtEnd)){
					
					PAContext.psaAnnual = PAContext.schedule.schedule(ScheduleParameters.createOneTime(Util.getCurrentTime(), ScheduleParameters.LAST_PRIORITY+1), u, "recordAnnualPSAOutput");
				}
				
			}
			
			public void resetClock(){
				this.clockYears = 0;
				this.clock3Months = 0;
				this.clockMonths = 0;
			}
			
			public double getClockYears(){
				return this.clockYears;
			}
			public void setClockYears(double clock3Months){
				this.clockYears = clock3Months/4;
			}
			public double getClock3Months(){
				return this.clock3Months;
			}
			public void setClock3Months(double clock3Months){
				this.clock3Months = clock3Months;
			}
			public double getClockMonths(){
				return this.clockMonths;
			}
			public void setClockMonths(double clock3Months){
				this.clockMonths = 3*clock3Months;
			}
			
		}
		
		//// EXAMPLE create random seeds sequence for replications 
		public static int[] createRandomSeed(int replications){
			int[] myRandomSeed = new int[replications];  
			Random myRandom = new Random(0);
			int i;
			for (i = 0; i<replications; i++){
				myRandomSeed[i] = myRandom.nextInt();
				
				if(myRandomSeed[i] < 0){
				
					myRandomSeed[i] = (-1)*myRandomSeed[i];
					}
			}
			return myRandomSeed;
		}

		public static double getCurrentTime(){
			return PAContext.schedule.getTickCount();
		}
		public static double getYears(Double time){
			double years = CONVERT_YEARS*time;
			return years;
		}
		public static double getMonths(Double time){
			double months = CONVERT_MONTHS*time;
			return months;
		}
		public static double getDays(Double time){
			double days = CONVERT_DAYS*time;
			return days;
		}
		
		//Formatting to two decimal points --- for exporting to text. the calculations happen before
		public static String formatNumber(double myDouble){
			DecimalFormat formatter = new DecimalFormat("###.##");
			String out = formatter.format(myDouble);
			return out;
		}
		
		//////return an array with all filenames of the same data files in a directory
		
		public static String[] getSameTypeDataFiles(String fileName, String directory){
			int numOfDataFiles = getNumOfFilesInData(fileName);
			String filenames[] = new String[numOfDataFiles];
		
			  for (int i = 0; i < numOfDataFiles; i++) {
			      filenames[i] = directory+fileName+(i+1)+".csv";
			 
			    }
			  return filenames; 	
		}
		
		
		//////return the number of files in a directory
		
		public static int getNumOfFilesInData(String fileName){
			int numOfDataFiles = 0; 
			
			File folder = new File(DataFiles.DATA_DIRECTORY.toString());
			File[] listOfFiles = folder.listFiles();
			
			
			  for (int i = 0; i < listOfFiles.length; i++) {
			      if (listOfFiles[i].isFile() & listOfFiles[i].getName().contains(fileName)) {
			    	 
			        numOfDataFiles++;
			      } 
			    }
			  return numOfDataFiles; 	
		}
		
		
		public static void psaErrorCheck(){
			
			if (psaError){
				//JOptionPane.showMessageDialog(null, psaErrorMessage+"The simulation will terminate.", "", JOptionPane.ERROR_MESSAGE);
			/*	JOptionPane.showMessageDialog(null, "<html><body><p style='width: 200px;'>"+psaErrorMessage+"</p>"+"<p style='font: 14pt;'>"+"<b>"+"The simulation will terminate."+"</b>"+"</p></body></html>", 
						"ERROR", JOptionPane.ERROR_MESSAGE);
				*/
				RSApplication.getRSApplicationInstance().getGui().getFrame().dispose();
			
			}
			else {
				individualResultsDialogue();
				try {
					createOutFiles();
				} catch (IOException e) {
					// Auto-generated catch block
					e.printStackTrace();
				}
			}
			/*
			JOptionPane.showMessageDialog(
				    this, 
				    "<html><body><p style='width: 200px;'>"+exp.getMessage()+"</p></body></html>", 
				    "Error", 
				    JOptionPane.ERROR_MESSAGE);
			
			*/
			
			
		}
		
		///// READ FROM DATA FILES TO HASHMAP AND CREATE RECORDS 

		public static void readPeopleDataFiles(DataFiles filename, Data ddata){
			int id = 0;
			Map peoplePropMap = new HashMap();
			
			if (filename == DataFiles.PEOPLE_DATA_FILE){
				String fn = filename.toString();
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
						}
						// Read lines, parse data and add to HasMap
						String line = null;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
							try {
								id++;
								int idx = 0;
								
								peoplePropMap.put(PeopleProperties.ID.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.AGE.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.GENDER.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.PA_STATUS.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.HAD_CVD.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.HAD_T2D.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.BMI.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.HDL_RATIO.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.S_BLOOD_PRESSURE.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.D_BLOOD_PRESSURE.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.SMOKING.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.ETHNICITY.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.FAMILY_CVD_HISTORY.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.DEPRIVATION_TOWN.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.BP_TREATMENT.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.RHEUMATOID_ARTHRITIS.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.CHRONIC_KIDNEY_DISEASE.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.ATRIAL_FIBRILLATION.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.HAD_T1D.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.CORT.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.FAMILY_T2D_HISTORY.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.EDUCATION.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.SAMPLE_WEIGHT.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.UTILITY_IMPEQ5D.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.DEPRIVATION_IMD.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.HAD_DEPRESSION.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.HAD_MSI.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.SEC.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.LLI.toString().trim(), data[idx++].trim());
								peoplePropMap.put(PeopleProperties.COMORBIDITY.toString().trim(), data[idx++].trim());
								
								RecordProperties peopleRecProp = new RecordProperties(peoplePropMap);
								ddata.addRecord(id, DataType.PEOPLE.toString(), peopleRecProp);
								
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
			}
			else{
				throw new IllegalArgumentException(
						"Error with the data files");
			}
		
		}
		
		///start read
		//parameters e.g.
		//filename DataFiles.INTERVENTIONS_DATA_FILE.toString()
		//directory DataFiles.DATA_DIRECTORY.toString()
		
		public static void readReplicationsDataFiles(String filename, String directory){
			
			
			
			String[] myFiles = Util.getSameTypeDataFiles(filename, directory);
			
			
			//Map<String, Map<String, ArrayList<String>>> replicationsData = new HashMap<String, Map<String, ArrayList<String>>>();
			//Map<String, ArrayList<String>> arrayMap = new HashMap<String, ArrayList<String>>();
			
			if(filename == DataFiles.EVENTS_DATA_FILE.toString()){
				int id =0;
				
				for(int j=0; j < myFiles.length; j++){
					
					id++;
					Map<String, Map<String, ArrayList<String>>> replicationsData = new HashMap<String, Map<String, ArrayList<String>>>();
					Map<String, ArrayList<String>> arrayMap = new HashMap<String, ArrayList<String>>();

					ArrayList<String> name = new ArrayList();
					ArrayList<String> acuteDuration = new ArrayList();
					ArrayList<String> acuteCost = new ArrayList();
					ArrayList<String> acuteUtility = new ArrayList();
					ArrayList<String> acutePA = new ArrayList();
					ArrayList<String> postAcuteCost = new ArrayList();
					ArrayList<String> postAcuteUtility = new ArrayList();
					ArrayList<String> postAcutePAInactiveMmean = new ArrayList();
					ArrayList<String> postAcutePAInactiveMsd = new ArrayList();
					ArrayList<String> postAcutePAModeratelyActiveMmean = new ArrayList();
					ArrayList<String> postAcutePAModeratelyActiveMsd = new ArrayList();
					ArrayList<String> postAcutePAVeryActiveMmean = new ArrayList();
					ArrayList<String> postAcutePAVeryActiveMsd = new ArrayList();
					ArrayList<String> postAcutePAInactiveFmean = new ArrayList();
					ArrayList<String> postAcutePAInactiveFsd = new ArrayList();
					ArrayList<String> postAcutePAModeratelyActiveFmean = new ArrayList();
					ArrayList<String> postAcutePAModeratelyActiveFsd = new ArrayList();
					ArrayList<String> postAcutePAVeryActiveFmean = new ArrayList();
					ArrayList<String> postAcutePAVeryActiveFsd = new ArrayList();
													
					String fn = myFiles[j];
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
						}
						// Read lines, parse data and add to HasMap
						String line = null;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
							try {
								
								int idx = 0;
								//increase the read index before write because in csv 1st column is the PSA number
								name.add(data[++idx].trim());
								acuteDuration.add(data[++idx].trim());
								acuteCost.add(data[++idx].trim());
								acuteUtility.add(data[++idx].trim());
								acutePA.add(data[++idx].trim());
								postAcuteCost.add(data[++idx].trim());
								postAcuteUtility.add(data[++idx].trim());
								postAcutePAInactiveMmean.add(data[++idx].trim());
								postAcutePAInactiveMsd.add(data[++idx].trim());
								postAcutePAModeratelyActiveMmean.add(data[++idx].trim());
								postAcutePAModeratelyActiveMsd.add(data[++idx].trim());
								postAcutePAVeryActiveMmean.add(data[++idx].trim());
								postAcutePAVeryActiveMsd.add(data[++idx].trim());
								postAcutePAInactiveFmean.add(data[++idx].trim());
								postAcutePAInactiveFsd.add(data[++idx].trim());
								postAcutePAModeratelyActiveFmean.add(data[++idx].trim());
								postAcutePAModeratelyActiveFsd.add(data[++idx].trim());
								postAcutePAVeryActiveFmean.add(data[++idx].trim());
								postAcutePAVeryActiveFsd.add(data[++idx].trim());
																
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
					
					///put
					//replicationsData.
					arrayMap.put(EventsProperties.NAME.toString().trim(), name);
					arrayMap.put(EventsProperties.ACUTE_DURATION_MONTHS.toString().trim(), acuteDuration);
					arrayMap.put(EventsProperties.ACUTE_COST_MONTH.toString().trim(), acuteCost);
					arrayMap.put(EventsProperties.ACUTE_UTILITY_MULTIPLIER.toString().trim(), acuteUtility);
					arrayMap.put(EventsProperties.ACUTE_PA.toString().trim(), acutePA);
					arrayMap.put(EventsProperties.POST_ACUTE_COST_MONTH.toString().trim(), postAcuteCost);
					arrayMap.put(EventsProperties.POST_ACUTE_UTILITY_MULTIPLIER.toString().trim(), postAcuteUtility);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_INACTIVE_M_MEAN.toString().trim(), postAcutePAInactiveMmean);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_INACTIVE_M_SD.toString().trim(), postAcutePAInactiveMsd);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_MODERATELY_ACTIVE_M_MEAN.toString().trim(), postAcutePAModeratelyActiveMmean);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_MODERATELY_ACTIVE_M_SD.toString().trim(), postAcutePAModeratelyActiveMsd);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_VERY_ACTIVE_M_MEAN.toString().trim(), postAcutePAVeryActiveMmean);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_VERY_ACTIVE_M_SD.toString().trim(), postAcutePAVeryActiveMsd);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_INACTIVE_F_MEAN.toString().trim(), postAcutePAInactiveFmean);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_INACTIVE_F_SD.toString().trim(), postAcutePAInactiveFsd);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_MODERATELY_ACTIVE_F_MEAN.toString().trim(), postAcutePAModeratelyActiveFmean);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_MODERATELY_ACTIVE_F_SD.toString().trim(), postAcutePAModeratelyActiveFsd);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_VERY_ACTIVE_F_MEAN.toString().trim(), postAcutePAVeryActiveFmean);
					arrayMap.put(EventsProperties.POST_ACUTE_PA_VERY_ACTIVE_F_SD.toString().trim(), postAcutePAVeryActiveFsd);
										
					if(name.size() < PAContext.replications){
						psaError = true;
						//psaErrorMessage = psaErrorMessage + name.get(0)+"; ";
						psaErrorMessage = psaErrorMessage + fn+"; ";
					}
					
					replicationsData.put(DataType.EVENT.toString(), arrayMap);
				
					//PAContext.events = new ReplicationData(replicationsData);
					//PAContext.replicationsList.addRecord(name.get(0), PAContext.events);
					PAContext.replicationsData.addRecord(id, DataType.EVENT.toString(), name.get(0), new ReplicationData(replicationsData));
				
				}
				
			}
			
			else if(filename == DataFiles.INTERVENTIONS_DATA_FILE.toString()){
				int id = 0;
				
				String sex;
				double minAge, maxAge, minPA, maxPA;
				if (PAContext.p.getValueAsString("igender").trim().equalsIgnoreCase("0")){sex = "ALL";}
				else if (PAContext.p.getValueAsString("igender").trim().equalsIgnoreCase("1")){sex = "M";}
				else {sex = "F";}
				minAge = PAContext.p.getDouble("minPAIntroAge");
				maxAge = PAContext.p.getDouble("maxPAIntroAge");
				minPA = PAContext.p.getDouble("minPAIntroStatus");
				maxPA = PAContext.p.getDouble("maxPAIntroStatus");
				
				for(int j=0; j < myFiles.length; j++){
					
					Map<String, Map<String, ArrayList<String>>> replicationsData = new HashMap<String, Map<String, ArrayList<String>>>();
					Map<String, ArrayList<String>> arrayMap = new HashMap<String, ArrayList<String>>();
					
					ArrayList<String> name = new ArrayList();
					ArrayList<String> introAgeMin = new ArrayList();
					ArrayList<String> introAgeMax = new ArrayList();
					ArrayList<String> introGender = new ArrayList();
					ArrayList<String> introPAMin = new ArrayList();
					ArrayList<String> introPAMax = new ArrayList();
					ArrayList<String> duration = new ArrayList();
					ArrayList<String> effectStartTime = new ArrayList();
					ArrayList<String> uptakeProbability = new ArrayList();
					ArrayList<String> effectOnPAmean = new ArrayList();
					ArrayList<String> effectOnPAsd = new ArrayList();
					ArrayList<String> healthFixedCost = new ArrayList();
					ArrayList<String> healthIndividualCostMonth = new ArrayList();
					ArrayList<String> sportFixedCost = new ArrayList();
					ArrayList<String> sportIndividualCostMonth = new ArrayList();
					ArrayList<String> educationFixedCost = new ArrayList();
					ArrayList<String> educationIndividualCostMonth = new ArrayList();
					ArrayList<String> transportFixedCost = new ArrayList();
					ArrayList<String> transportIndividualCostMonth = new ArrayList();
					ArrayList<String> localAuthoritiesFixedCost = new ArrayList();
					ArrayList<String> localAuthoritiesIndividualCostMonth = new ArrayList();
					ArrayList<String> otherSectorFixedCost = new ArrayList();
					ArrayList<String> otherSectorIndividualCostMonth = new ArrayList();
					
					String fn = myFiles[j];
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
						}
						// Read lines, parse data and add to HasMap
						String line = null;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
							try {
								
								int idx = 0;
								//increase the read index before write because in csv 1st column is the PSA number
								name.add(data[++idx].trim());
								introAgeMin.add(data[++idx].trim());
								introAgeMax.add(data[++idx].trim());
								introGender.add(data[++idx].trim());
								introPAMin.add(data[++idx].trim());
								introPAMax.add(data[++idx].trim());
								duration.add(data[++idx].trim());
								effectStartTime.add(data[++idx].trim());
								uptakeProbability.add(data[++idx].trim());
								effectOnPAmean.add(data[++idx].trim());
								effectOnPAsd.add(data[++idx].trim());
								healthFixedCost.add(data[++idx].trim());
								healthIndividualCostMonth.add(data[++idx].trim());
								sportFixedCost.add(data[++idx].trim());
								sportIndividualCostMonth.add(data[++idx].trim());
								educationFixedCost.add(data[++idx].trim());
								educationIndividualCostMonth.add(data[++idx].trim());
								transportFixedCost.add(data[++idx].trim());
								transportIndividualCostMonth.add(data[++idx].trim());
								localAuthoritiesFixedCost.add(data[++idx].trim());
								localAuthoritiesIndividualCostMonth.add(data[++idx].trim());
								otherSectorFixedCost.add(data[++idx].trim());
								otherSectorIndividualCostMonth.add(data[++idx].trim());
																
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
					
					///put
					//replicationsData.
					
					if (Double.valueOf(introAgeMin.get(0))>=minAge && Double.valueOf(introAgeMax.get(0))<=maxAge &&
						Double.valueOf(introPAMin.get(0))>=minPA && Double.valueOf(introPAMax.get(0))<=maxPA &&
						sex.equalsIgnoreCase(introGender.get(0))){
						
						++interventionCount;
						totalHealthFixedCost += Double.parseDouble(healthFixedCost.get(0));
						totalSportFixedCost += Double.parseDouble(sportFixedCost.get(0));		
						totalEducationFixedCost += Double.parseDouble(educationFixedCost.get(0));
						totalTransportFixedCost += Double.parseDouble(transportFixedCost.get(0));
						totalLocalAuthFixedCost += Double.parseDouble(localAuthoritiesFixedCost.get(0));
						totalOtherFixedCost += Double.parseDouble(otherSectorFixedCost.get(0));
						totalIntrventionsFixedCost = totalHealthFixedCost + totalSportFixedCost + totalEducationFixedCost
						+ totalTransportFixedCost +totalLocalAuthFixedCost + totalOtherFixedCost;
						
						id++;
						arrayMap.put(InterventionsProperties.NAME.toString().trim(), name);
						arrayMap.put(InterventionsProperties.INTRO_AGE_MIN.toString().trim(), introAgeMin);
						arrayMap.put(InterventionsProperties.INTRO_AGE_MAX.toString().trim(), introAgeMax);
						arrayMap.put(InterventionsProperties.INTRO_GENDER.toString().trim(), introGender);
						arrayMap.put(InterventionsProperties.INTRO_PA_STATUS_MIN.toString().trim(), introPAMin);
						arrayMap.put(InterventionsProperties.INTRO_PA_STATUS_MAX.toString().trim(), introPAMax);
						arrayMap.put(InterventionsProperties.DURATION_MONTHS.toString().trim(), duration);
						arrayMap.put(InterventionsProperties.EFFECT_START_TIME_MONTHS.toString().trim(), effectStartTime);
						arrayMap.put(InterventionsProperties.UPTAKE_PROBABILITY.toString().trim(), uptakeProbability);
						arrayMap.put(InterventionsProperties.EFFECT_ON_PA_MEAN.toString().trim(), effectOnPAmean);
						arrayMap.put(InterventionsProperties.EFFECT_ON_PA_SD.toString().trim(), effectOnPAsd);
						arrayMap.put(InterventionsProperties.HEALTH_SECTOR_FIXED_COST.toString().trim(), healthFixedCost);
						arrayMap.put(InterventionsProperties.HEALTH_SECTOR_INDIVIDUAL_COST_PER_MONTH.toString().trim(), healthIndividualCostMonth);
						arrayMap.put(InterventionsProperties.SPORT_SECTOR_FIXED_COST.toString().trim(), sportFixedCost);
						arrayMap.put(InterventionsProperties.SPORT_SECTOR_INDIVIDUAL_COST_PER_MONTH.toString().trim(), sportIndividualCostMonth);
						arrayMap.put(InterventionsProperties.EDUCATION_SECTOR_FIXED_COST.toString().trim(), educationFixedCost);
						arrayMap.put(InterventionsProperties.EDUCATION_SECTOR_INDIVIDUAL_COST_PER_MONTH.toString().trim(), educationIndividualCostMonth);
						arrayMap.put(InterventionsProperties.TRANSPORT_SECTOR_FIXED_COST.toString().trim(), transportFixedCost);
						arrayMap.put(InterventionsProperties.TRANSPORT_SECTOR_INDIVIDUAL_COST_PER_MONTH.toString().trim(), transportIndividualCostMonth);
						arrayMap.put(InterventionsProperties.LOCAL_AUTHORITIES_FIXED_COST.toString().trim(), localAuthoritiesFixedCost);
						arrayMap.put(InterventionsProperties.LOCAL_AUTHORITIES_INDIVIDUAL_COST_PER_MONTH.toString().trim(), localAuthoritiesIndividualCostMonth);
						arrayMap.put(InterventionsProperties.OTHER_SECTOR_FIXED_COST.toString().trim(), otherSectorFixedCost);
						arrayMap.put(InterventionsProperties.OTHER_SECTOR_INDIVIDUAL_COST_PER_MONTH.toString().trim(), otherSectorIndividualCostMonth);
						
						if(name.size() < PAContext.replications){
							psaError = true;
							//psaErrorMessage = psaErrorMessage + name.get(0)+"; ";
							psaErrorMessage = psaErrorMessage + fn+"; ";
						}
						
						replicationsData.put(DataType.INTERVENTION.toString(), arrayMap);
						
						
						PAContext.replicationsData.addRecord(id, DataType.INTERVENTION.toString(), name.get(0), new ReplicationData(replicationsData));
						
						
					}
						
					
				}
			}
			
			else{
				throw new IllegalArgumentException(
						"Error with the data files");
			}
			
		}

		
		public static void readRiskCoef(DataFiles filename){
			
			if (filename == DataFiles.QRISK_COEF_DATA_FILE){
				String fn = filename.toString().trim();
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
						}
						// Read lines, parse data and add to HasMap
						String line = null;
						int replIdx = 0;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
							try {
								int dataIdx = 0;
								int varIdx = 0;
								int matrixIdx = 0;
								int riskIdx = 0;
								
								while (dataIdx<1){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<8){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<18){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<23){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<30){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<37){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<42){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<52){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<57){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<67){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								matrixIdx=0;
								++riskIdx;
								while (dataIdx<(67+1)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+8)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+18)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+23)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+30)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+37)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+42)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+52)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+57)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(67+67)){
									PAContext.qRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								///////////
								replIdx++;
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
					
					if(PAContext.qRiskCoef.get(0).get(0).get(0).size() < PAContext.replications){
						psaError = true;
						psaErrorMessage = psaErrorMessage + filename+"; ";
					}
					
			}
			
			else if (filename == DataFiles.DRISK_COEF_DATA_FILE){
				String fn = filename.toString().trim();
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
						}
						// Read lines, parse data and add to HasMap
						String line = null;
						int replIdx = 0;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
							try {
								int dataIdx = 0;
								int varIdx = 0;
								int matrixIdx = 0;
								int riskIdx = 0;
								
								while (dataIdx<1){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<6){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<16){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<21){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<26){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<30){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<33){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<36){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								
								varIdx=0;
								matrixIdx=0;
								++riskIdx;
								
								while (dataIdx<(36+1)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+6)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+16)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+21)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+26)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+30)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+33)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								varIdx=0;
								++matrixIdx;
								while (dataIdx<(36+36)){
									PAContext.dRiskCoef.addToReplicationArray(riskIdx, matrixIdx, varIdx++, replIdx, data[++dataIdx]);
								}
								///////////
								replIdx++;
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
					if(PAContext.qRiskCoef.get(0).get(0).get(0).size() < PAContext.replications){
						psaError = true;
						psaErrorMessage = psaErrorMessage + filename+"; ";
					}
			}
			else{
				throw new IllegalArgumentException(
						"Error with the data files");
			}
			
		}
		
		
		
		public static void read2DArrayData(DataFiles filename, Util.My2DArray<Double> my2D){
			
		
				String fn = filename.toString().trim();
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
						}
						// Read lines, parse data and add to HashMap
						String line = null;
						int replIdx = 0;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
						//	System.out.println(data.length);
							try {
								int dataIdx = 0;
								int ageIdx = 0;
							while (dataIdx<data.length-1){
								my2D.addToInnerArray(ageIdx++, replIdx, Double.valueOf(data[++dataIdx]));
															
							}
								///////////
								replIdx++;
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
					
					if(my2D.get(0).size() < PAContext.replications){
						psaError = true;
						psaErrorMessage = psaErrorMessage + filename+"; ";
					}
							
		}
		
		
		
		///Read mortality annual probability from life tables and then read the relative risks of conditions
		//the 3-months mortality probability for healthy or for any condition is calculated in the model
	
		public static void readMortality3monthsProbability(DataFiles filename){
			
		
				String fn = filename.toString().trim();
					BufferedReader br = null;
					try {
						br = new BufferedReader(new FileReader(fn));
						// Skip header
						for (int i = 0; i < DATA_FILES_HEADER_LINES; ++i) {
							br.readLine();
							
						}
						// Read lines, parse data and add to HashMap
						String line = null;
						int lineCount =0;
						while ((line = br.readLine()) != null) {
							// Split the line around commas
							final String[] data = line.split(",");
							// Check if current line seems all right
							/*if (data.length != 16) { // *** length of data 
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename.name(), line));
							}*/
						//	System.out.println(data.length);
							try {
								//System.out.println(lineCount);
							if(lineCount ==0){	
								int dataIdx = 0;
								while (dataIdx<data.length-1){
									PAContext.mortality3monthsMaleHealthyNonCVD[dataIdx] = Double.valueOf(data[++dataIdx]);
								}
							}
							else if(lineCount ==1){	
								int dataIdx = 0;
								while (dataIdx<data.length-1){
									PAContext.mortality3monthsMaleHealthyCVD[dataIdx] = Double.valueOf(data[++dataIdx]);
								}
							}
							else if(lineCount ==2){	
								int dataIdx = 0;
								while (dataIdx<data.length-1){
									PAContext.mortality3monthsFemaleHealthyNonCVD[dataIdx] = Double.valueOf(data[++dataIdx]);
								}
							}
							else if(lineCount ==3){	
								int dataIdx = 0;
								while (dataIdx<data.length-1){
									PAContext.mortality3monthsFemaleHealthyCVD[dataIdx] = Double.valueOf(data[++dataIdx]);
								}
							}
							
								///////////
							lineCount++;	
							} catch (final NumberFormatException e) {
								e.printStackTrace();
								throw new IllegalArgumentException(String.format(
										"File %s contains a malformed input line: %s",
										filename, line), e);
							}
						}
					} catch (final FileNotFoundException e) {
						e.printStackTrace();
					} catch (final IOException e) {
						e.printStackTrace();
					} finally {
						if (br != null) {
							try {
								br.close();
							} catch (final IOException e) {
								e.printStackTrace();
							}
						}
					}
		
		}
		
	/////END READ
	
		// range selection variables (eg age, bmi, pa are in Data class) 
		public static List selectPopulation(Data d){
			Map properties = new HashMap();
			
			if((PAContext.p.getValueAsString("gender").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.GENDER.toString().trim(), "M");
			}
			else if((PAContext.p.getValueAsString("gender").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.GENDER.toString().trim(), "F");
			}
	
			if((PAContext.p.getValueAsString("cvd").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.HAD_CVD.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("cvd").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.HAD_CVD.toString().trim(), "1");
			}
						
			if((PAContext.p.getValueAsString("t2d").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.HAD_T2D.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("t2d").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.HAD_T2D.toString().trim(), "1");
			}
			
			if((PAContext.p.getValueAsString("t1d").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.HAD_T1D.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("t1d").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.HAD_T1D.toString().trim(), "1");
			}
			
			if((PAContext.p.getValueAsString("depression").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.HAD_DEPRESSION.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("depression").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.HAD_DEPRESSION.toString().trim(), "1");
			}
			
			if((PAContext.p.getValueAsString("msi").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.HAD_MSI.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("msi").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.HAD_MSI.toString().trim(), "1");
			}
			
			if((PAContext.p.getValueAsString("bpTreatment").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.BP_TREATMENT.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("bpTreatment").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.BP_TREATMENT.toString().trim(), "1");
			}
			
			if((PAContext.p.getValueAsString("kidneyDisease").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.CHRONIC_KIDNEY_DISEASE.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("kidneyDisease").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.CHRONIC_KIDNEY_DISEASE.toString().trim(), "1");
			}
			
			if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("1"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "0");
			}
			else if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("2"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "1");
			}
			else if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("3"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "2");
			}
			else if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("4"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "3.1");
			}
			else if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("5"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "3.2");
			}
			else if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("6"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "4");
			}
			else if((PAContext.p.getValueAsString("sec").trim().equalsIgnoreCase("7"))){
				properties.put(PeopleProperties.SEC.toString().trim(), "5");
			}
			
			
			RecordProperties selectedProp = new RecordProperties(properties);
			

			List matchingRecords = d.search(selectedProp);
			/*
			System.out.print("matchingRecords.size() ");
			System.out.println(matchingRecords.size());
			*/
			PAContext.totalPopulation = matchingRecords.size();
		/*	if(matchingRecords.size()==0){
				JOptionPane.showMessageDialog(null, "There is no record that matches your criteria. The simulation will terminate.");
				PAContext.enoughCohort = false;
				RSApplication.getRSApplicationInstance().getGui().getFrame().dispose();
				//RunEnvironment.getInstance().endRun();
			}
			else{JOptionPane.showMessageDialog(null, "Numbers of records matching your criteria: "+matchingRecords.size());}
			*/
			return matchingRecords;
			
		}
		
		
		public static void initialiseAgentsFromData(Agent a, Record record){
			a.setID(Integer.parseInt(record.getProp().getProperty(PeopleProperties.ID.toString()).toString()));
			a.setAge(Double.parseDouble(record.getProp().getProperty(PeopleProperties.AGE.toString()).toString()));
			a.setGender(record.getProp().getProperty(PeopleProperties.GENDER.toString()).toString());
			a.setPAStatus(Double.parseDouble(record.getProp().getProperty(PeopleProperties.PA_STATUS.toString()).toString()));
			a.setHadCVD(Integer.parseInt(record.getProp().getProperty(PeopleProperties.HAD_CVD.toString()).toString()));
			a.setHadT2D(Integer.parseInt(record.getProp().getProperty(PeopleProperties.HAD_T2D.toString()).toString()));
			a.setBMI(Double.parseDouble(record.getProp().getProperty(PeopleProperties.BMI.toString()).toString()));
			a.setHDLRatio(Double.parseDouble(record.getProp().getProperty(PeopleProperties.HDL_RATIO.toString()).toString()));
			a.setSBloodPressure(Double.parseDouble(record.getProp().getProperty(PeopleProperties.S_BLOOD_PRESSURE.toString()).toString()));
			a.setDBloodPressure(Double.parseDouble(record.getProp().getProperty(PeopleProperties.D_BLOOD_PRESSURE.toString()).toString()));
			a.setSmoking(Integer.parseInt(record.getProp().getProperty(PeopleProperties.SMOKING.toString()).toString()));
			a.setEthnicity(record.getProp().getProperty(PeopleProperties.ETHNICITY.toString()).toString());
			a.setFamilyCVDHistory(Integer.parseInt(record.getProp().getProperty(PeopleProperties.FAMILY_CVD_HISTORY.toString()).toString()));
			a.setDeprivationTown(Double.parseDouble(record.getProp().getProperty(PeopleProperties.DEPRIVATION_TOWN.toString()).toString()));
			a.setBpTreatment(Integer.parseInt(record.getProp().getProperty(PeopleProperties.BP_TREATMENT.toString()).toString()));
			a.setRheumatoidArthritis(Integer.parseInt(record.getProp().getProperty(PeopleProperties.RHEUMATOID_ARTHRITIS.toString()).toString()));
			a.setCHronicKidneyDisease(Integer.parseInt(record.getProp().getProperty(PeopleProperties.CHRONIC_KIDNEY_DISEASE.toString()).toString()));
			a.setAtrialFibrillation(Integer.parseInt(record.getProp().getProperty(PeopleProperties.ATRIAL_FIBRILLATION.toString()).toString()));
			a.setHadT1D(Integer.parseInt(record.getProp().getProperty(PeopleProperties.HAD_T1D.toString()).toString()));
			a.setCort(Integer.parseInt(record.getProp().getProperty(PeopleProperties.CORT.toString()).toString()));
			a.setFamilyT2DHistory(Integer.parseInt(record.getProp().getProperty(PeopleProperties.FAMILY_T2D_HISTORY.toString()).toString()));
			a.setEducation(Integer.parseInt(record.getProp().getProperty(PeopleProperties.EDUCATION.toString()).toString()));
			a.setSampleWeight(Double.parseDouble(record.getProp().getProperty(PeopleProperties.SAMPLE_WEIGHT.toString()).toString()));
			a.setUtilityIMPEQ5D(Double.parseDouble(record.getProp().getProperty(PeopleProperties.UTILITY_IMPEQ5D.toString()).toString()));
			a.setDeprivationIMD(Double.parseDouble(record.getProp().getProperty(PeopleProperties.DEPRIVATION_IMD.toString()).toString()));
			a.setHadDepression(Integer.parseInt(record.getProp().getProperty(PeopleProperties.HAD_DEPRESSION.toString()).toString()));
			a.setHadMSI(Integer.parseInt(record.getProp().getProperty(PeopleProperties.HAD_MSI.toString()).toString()));
			a.setSocioEconomicClass(Double.parseDouble(record.getProp().getProperty(PeopleProperties.SEC.toString()).toString()));
			a.setLongLastingIllness(Integer.parseInt(record.getProp().getProperty(PeopleProperties.LLI.toString()).toString()));
			a.setComorbidity(Integer.parseInt(record.getProp().getProperty(PeopleProperties.COMORBIDITY.toString()).toString()));
			
			a.setAgeAtEntry(a.getAge());
			a.setPAStatusAtEntry(a.getPAStatus());
			a.setBMIAtEntry(a.getBMI());
			a.setHDLRatioAtEntry(a.getHDLRatio());
			a.setSBPAtEntry(a.getSBloodPressure());
			a.setUtilityAtEntry(a.getUtilityIMPEQ5D());
			a.setBaselineUtility(a.getUtilityIMPEQ5D());
			a.setCurrentUtility(a.getBaselineUtility());
			a.setPAPreviousPeriod(a.getPAStatus());
			
					
			////TODO 
			for (Iterator i = PAContext.replicationsData.getDataList().iterator();i.hasNext();){ 
				Record rec = (Record) i.next();
				if(rec.getType().equalsIgnoreCase(DataType.EVENT.toString())){
				

					
				}
				else if (rec.getType().equalsIgnoreCase(DataType.INTERVENTION.toString())){
					
				}
				
			}
			////TODO 
			
			
		}
		
		public static void resetAgent(Agent a){
			
			a.setIsDead(false);
			a.setCauseOfDeath(null);
			a.setCheckPAonUtilityEffect(true);			
			a.setInAcutePeriod(false);
			
			
			a.setLifeYears(0.0);
			a.setYearsActive(0.0);
			a.setTotalCost(0.0);
			a.setTotalCostDiscounted(0.0);
			a.setTotalEventCost(0.0);
			a.setTotalEventCostDiscounted(0.0);
			a.setTotalInterventionCost(0.0);
			a.setTotalInterventionCostDiscounted(0.0);
			a.setTotalQALYs(0.0);
			a.setTotalQALYsDiscounted(0.0);
			a.setTotalInterventionCostHealth(0.0);
			a.setTotalInterventionCostHealthDiscounted(0.0);
			a.setTotalInterventionCostSport(0.0);
			a.setTotalInterventionCostSportDiscounted(0.0); 
			a.setTotalInterventionCostEducation(0.0);
			a.setTotalInterventionCostEducationDiscounted(0.0); 
			a.setTotalInterventionCostTransport(0.0);
			a.setTotalInterventionCostTransportDiscounted(0.0); 
			a.setTotalInterventionCostOther(0.0);
			a.setTotalInterventionCostOtherDiscounted(0.0);
			
			a.setTotalNoOfEvents(0);
			a.setTotalNoOfCVD(0);
			a.setNoOfMSI(0);
			a.setNoOfDprsEpisodes(0);
			a.setNoOfStroke(0);
			a.setNoOfCHD(0);
			a.setNoOfT2D(0);
			a.setNoOfInterventionsIntro(0);
			a.setNoOfInterventionsUptake(0);
					
			//a.clearEventHistory();
			//a.clearInterventionIntroducedHistory();
			//a.clearInterventionUptakeHistory();
						
			a.clearCHDEventsList();
			a.clearStrokeEventsList();
			a.clearT2DEventsList();
			a.clearMSIEventsList();
			a.clearDepressionEventsList();
			a.clearInterventionsIntroducedList();
			a.clearInterventionsUptakeList();
					
		}
				
		public static void individualResultsDialogue(){
			
			Object[] options = {"Yes", "No"};
			/*
			int dialogue = JOptionPane.showOptionDialog(null, "Do you want to write results for each individual?", "Individual results", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[1]);

			if (dialogue == JOptionPane.YES_OPTION){
				//System.out.println("Dialogue: Yes");
				PAContext.individualRecordOn = true; 
			}*/
			/*
			else{
				System.out.println("Dialogue: No");
			}
			*/
		}
		
		public static void createOutFiles() throws IOException{
			dateAppend = new SimpleDateFormat("yyyyMMddhhmmss").format(new Date());
			
			currentOutDirectory = new File("output/"+dateAppend);
			currentOutDirectory.mkdir();
			
			/////////////INDIVIDUAL TRACK
			String outfile = currentOutDirectory.getPath() + OutputFiles.INDIVIDUAL.toString().trim()+"TRACK.csv";
			File f2 = new File(outfile);
			if(!f2.exists()){
				try{
					f2.createNewFile();
					FileWriter fw = new FileWriter(outfile, true);
					BufferedWriter bw = new BufferedWriter(fw);
					bw.write("Simulation Time"+","+"ID in model"+","+"ID in DB"+","+"Age"+","+"BMI"+","+"PA Status"+","+"PA previous period"+","+"Is in acute period?"+","+"Event name Stroke"
					+","+"Number of Stroke Events"+","+"Stroke start time"+","+"Age at stroke start"+","+"PA at stroke start"+","+"Stroke acute duration (months)"
					+","+"Stroke acute PA status"+","+"Event name CHD"+","+"Number of CHD Events"+","+"CHD start time"+","+"Age at CHD start"+","+"PA at CHD start"
					+","+"CHD acute duration (months)"+","+"CHD acute PA status"+","+"Event name Depression"+","+"Number of Depression Events"+","+"Depression start time"
					+","+"Age at Depression start"+","+"PA at depression start"+","+"depression acute duration (months)"+","+"depression acute PA status"
					+","+"Event name MSI"+","+"Number of MSI Events"+","+"MSi start time"+","+"Age at MSI start"+","+"PA at MSI start"+","+"MSI acute duration (months)"
					+","+"MSI acute PA status"+","+"Event name T2D"+","+"Number of T2D Events"+","+"T2D start time"+","+"Age at T2D start"+","+"PA at T2D start"+","+"T2D acute duration (months)"
					+","+"T2D acute PA status\r\n");
					
					bw.close();
									
				}catch (IOException e) {
					// Auto-generated catch block
					e.printStackTrace();
				}
				
				
			}
			/////////////////INDIVIDUAL TRACK
			
			for (OutputFiles o : OutputFiles.values() ){
				String out = currentOutDirectory.getPath().trim() + o.toString().trim()+dateAppend+".csv";
				
				if(!PAContext.individualRecordOn && out.contains("Individual")){
				}
				else{
					//if(!out.contains("Annual")){File f1 = new File(out);
					
					File f1 = new File(out);
					if(!f1.exists()) {
					    try {
							f1.createNewFile();
							if(f1.getName().contains("Individual")){
							 	if(f1.getName().contains("Annual")){
								/*	FileWriter fw = new FileWriter(out, true);
									BufferedWriter bw = new BufferedWriter(fw);
									bw.write("SimYear"+","+"PSA"+","+"ID in model"+","+"ID in DB"+","+"Age at entry"+","+"Age at sim year"+","+"Life years"
									+","+"PA status at entry"+","+"PA status at sim year"+","+"Years active"+","+"Total No of events"
									+","+"Total No of CVD events"+","+"Total No of MSI events"+","+"Total No of Depression episodes"+","+"No of Strokes"+","+"No of CHD"
									+","+"Has T2D"+","+"Name of Interventions introduced"+","+"No of Interventions introduced"
									+","+"Name of Interventions uptake"+","+"No of Interventions uptake"+","+"Total QALYs"+","+"Total QALYs discounted"+","+"Total cost"
									+","+"Total cost discounted"+","+"Total event cost"+","+"Total event cost discounted"+","+"Total intervention cost"
									+","+"Total intervention cost discounted"+","+"Total intervention cost (health)"+","+"Total intervention cost (health) discounted"
									+","+"Total intervention cost (sport)"+","+"Total intervention cost (sport) discounted"
									+","+"Total intervention cost (education)"+","+"Total intervention cost (education) discounted"
									+","+"Total intervention cost (transport)"+","+"Total intervention cost (transport) discounted\r\n");
									bw.close();*/
									}
								else{
									FileWriter fw = new FileWriter(out, true);
									BufferedWriter bw = new BufferedWriter(fw);
									bw.write("PSA"+","+"ID in model"+","+"ID in DB"+","+"Age at entry"+","+"Age at sim end"+","+"Age at death"+","+"Cause of death"+","+"Life years"
									+","+"PA status at entry"+","+"PA status at sim end"+","+"PA status at death"+","+"BMI at entry"+","+"BMI at end"
									+","+"HDL Ratio at entry"+","+"HDL Ratio at end"+","+"SBP at entry"+","+"SBP at end"+","+"Utility at entry"+","+"Utility at end"
									+","+"Years active"+","+"Total No of events"+","+"Total No of CVD events"+","+"Total No of MSI events"
									+","+"Total No of Depression episodes"+","+"No of Strokes"+","+"No of CHD"+","+"Has T2D"+","+"Had T2D"+","+"Had T1D"+","+"Had CVD"
									+","+"Had MSI"+","+"Had Depression"+","+"SocioEconomicClass"
									+","+"Name of Interventions introduced"+","+"No of Interventions introduced"
									+","+"Name of Interventions uptake"+","+"No of Interventions uptake"+","+"Total QALYs"+","+"Total QALYs discounted"+","+"Total cost"
									+","+"Total cost discounted"+","+"Total event cost"+","+"Total event cost discounted"+","+"Total intervention cost"
									+","+"Total intervention cost discounted"+","+"Total intervention cost (health)"+","+"Total intervention cost (health) discounted"
									+","+"Total intervention cost (sport)"+","+"Total intervention cost (sport) discounted"
									+","+"Total intervention cost (education)"+","+"Total intervention cost (education) discounted"
									+","+"Total intervention cost (transport)"+","+"Total intervention cost (transport) discounted"
									+","+"Total intervention cost (local authorities)"+","+"Total intervention cost (local authorities) discounted"
									+","+"Total intervention cost (other)"+","+"Total intervention cost (other) discounted\r\n");
									bw.close();
									}
							 	 
								}
							else if(f1.getName().contains("PSA")){
							 	if(f1.getName().contains("Annual")){
									FileWriter fw = new FileWriter(out, true);
									BufferedWriter bw = new BufferedWriter(fw);
									bw.write("SimYear"+","+"PSA"+","+"Cohort size"
									+","+"Total life years"+","+"Avg life years"+","+"SD life years"
									+","+"Total PA status"+","+"Avg PA status"+","+"SD PA status"
									+","+"Total BMI"+","+"Avg BMI status"+","+"SD BMI"
									+","+"Total HDL ratio"+","+"Avg HDL ratio"+","+"SD HDL ratio"
									+","+"Total sbp"+","+"Avg sbp"+","+"SD sbp"
									+","+"Total utility"+","+"Avg utility"+","+"SD utility"
									+","+"Total events"+","+"Avg events"+","+"SD events"
									+","+"Total CVD events"+","+"Avg CVD events"+","+"SD CVD events"
									+","+"Total MSI events"+","+"Avg MSI events"+","+"SD MSI events"
									+","+"Total Depression episodes"+","+"Avg Depression episodes"+","+"SD Depression episodes"
									+","+"Total strokes"+","+"Avg strokes"+","+"SD strokes"+","+"Total CHD"+","+"Avg CHD"+","+"SD CHD"
									+","+"Total T2D"+","+"Avg T2D"+","+"SD T2D"
									+","+"Total interventions introduced"+","+"Avg interventions introduced"+","+"SD interventions introduced"
									+","+"Total interventions uptake"+","+"Avg interventions uptake"+","+"SD interventions uptake"
									+","+"Total years active"+","+"Avg years active"+","+"SD years active"
									+","+"Total cost"+","+"Avg cost"+","+"SD cost"
									+","+"Total event cost"+","+"Avg event cost"+","+"SD event cost"
									+","+"Total intervention cost"+","+"Avg intervention cost"+","+"SD intervention cost"
									+","+"Total QALYs"+","+"Avg QALYs"+","+"SD QALYs"
									+","+"Total intervention cost (health)"+","+"Avg intervention cost (health)"+","+"SD intervention cost (health)"
									+","+"Total intervention cost (sport)"+","+"Avg intervention cost (sport)"+","+"SD intervention cost (sport)"
									+","+"Total intervention cost (education)"+","+"Avg intervention cost (education)"+","+"SD intervention cost (education)"
									+","+"Total intervention cost (transport)"+","+"Avg intervention cost (transport)"+","+"SD intervention cost (transport)"
									+","+"Total intervention cost (local authorities)"+","+"Avg intervention cost (local authorities)"+","+"SD intervention cost (local authorities)"
									+","+"Total intervention cost (other)"+","+"Avg intervention cost (other)"+","+"SD intervention cost (other)\r\n");
									bw.close();
									}
								else{
									FileWriter fw = new FileWriter(out, true);
									BufferedWriter bw = new BufferedWriter(fw);
									bw.write("PSA"+","+"Cohort size"+","+"Avg life years"+","+"SD life years"+","+"Avg events"+","+"SD events"+","+"Avg CVD events"
									+","+"SD CVD events"+","+"Avg MSI events"+","+"SD MSI events"+","+"Avg Depression episodes"+","+"SD Depression episodes"
									+","+"Avg strokes"+","+"SD strokes"+","+"Avg CHD"+","+"SD CHD"+","+"Avg T2D"+","+"SD T2D"+","+"Avg interventions introduced"
									+","+"SD interventions introduced"+","+"Avg interventions uptake"+","+"SD interventions uptake"+","+"Avg years active"
									+","+"SD years active"+","+"Avg cost"+","+"SD cost"+","+"Avg cost discounted"+","+"SD cost discounted"+","+"Avg event cost"
									+","+"SD event cost"+","+"Avg event cost discounted"+","+"SD event cost discounted"+","+"Avg intervention cost"+","+"SD intervention cost"
									+","+"Avg intervention cost discounted"+","+"SD intervention cost discounted"+","+"Avg QALYs"+","+"SD QALYs"+","+"Avg QALYs discounted"
									+","+"SD QALYs discounted"+","+"Avg intervention cost (health)"+","+"SD intervention cost (health)"+","+"Avg intervention cost (health) discounted"
									+","+"SD intervention cost (health) discounted"+","+"Avg intervention cost (sport)"+","+"SD intervention cost (sport)"+","+"Avg intervention cost (sport) discounted"
									+","+"SD intervention cost (sport) discounted"+","+"Avg intervention cost (education)"+","+"SD intervention cost (education)"
									+","+"Avg intervention cost (education) discounted"+","+"SD intervention cost (education) discounted"+","+"Avg intervention cost (transport)"
									+","+"SD intervention cost (transport)"+","+"Avg intervention cost (transport) discounted"+","+"SD intervention cost (transport) discounted"
									+","+"Avg intervention cost (local authorities)"+","+"SD intervention cost (local authorities)"
									+","+"Avg intervention cost (local authorities) discounted"+","+"SD intervention cost (local authorities) discounted"
									+","+"Avg intervention cost (other)"+","+"SD intervention cost (other)"
									+","+"Avg intervention cost (other) discounted"+","+"SD intervention cost (other) discounted"
									+","+"Global number of interventions"+","+"Total fixed interventions cost"+","+"Fixed interventions cost (health)"
									+","+"Fixed interventions cost (sport)"+","+"Fixed interventions cost (education)"+","+"Fixed interventions cost (transport)"
									+","+"Fixed interventions cost (local authorities)"+","+"Fixed interventions cost (other sector)\r\n");
									bw.close();
									}
									
								}
							else if(f1.getName().contains("Strategy")){
								if(f1.getName().contains("Annual")){
								/*	FileWriter fw = new FileWriter(out, true);
									BufferedWriter bw = new BufferedWriter(fw);
									bw.write("SimYear"+","+"No of PSA"+","+"Cohort size"+","+"Avg life years"+","+"SD life years"+","+"Avg events"+","+"SD events"+","+"Avg CVD events"
											+","+"SD CVD events"+","+"Avg MSI events"+","+"SD MSI events"+","+"Avg Depression episodes"+","+"SD Depression episodes"
											+","+"Avg strokes"+","+"SD strokes"+","+"Avg CHD"+","+"SD CHD"
											+","+"Avg T2D"+","+"SD T2D"+","+"Avg interventions introduced"+","+"SD interventions introduced"+","+"Avg interventions uptake"
											+","+"SD interventions uptake"+","+"Avg years active"+","+"SD years active"+","+"Avg cost"+","+"SD cost"+","+"Avg cost discounted"
											+","+"SD cost discounted"+","+"Avg event cost"+","+"SD event cost"+","+"Avg event cost discounted"+","+"SD event cost discounted"
											+","+"Avg intervention cost"+","+"SD intervention cost"+","+"Avg intervention cost discounted"+","+"SD intervention cost discounted"
											+","+"Avg QALYs"+","+"SD QALYs"+","+"Avg QALYs discounted"+","+"SD QALYs discounted"+","+"Avg intervention cost (health)"
											+","+"SD intervention cost (health)"+","+"Avg intervention cost (health) discounted"+","+"SD intervention cost (health) discounted"
											+","+"Avg intervention cost (sport)"+","+"SD intervention cost (sport)"+","+"Avg intervention cost (sport) discounted"
											+","+"SD intervention cost (sport) discounted"+","+"Avg intervention cost (education)"+","+"SD intervention cost (education)"
											+","+"Avg intervention cost (education) discounted"+","+"SD intervention cost (education) discounted"+","+"Avg intervention cost (transport)"
											+","+"SD intervention cost (transport)"+","+"Avg intervention cost (transport) discounted"+","+"SD intervention cost (transport) discounted\r\n");
									bw.close();*/
									}
								else{
									FileWriter fw = new FileWriter(out, true);
									BufferedWriter bw = new BufferedWriter(fw);
									bw.write("No of PSA"+","+"Cohort size"+","+"Avg life years"+","+"SD life years"+","+"Avg events"+","+"SD events"+","+"Avg CVD events"
											+","+"SD CVD events"+","+"Avg MSI events"+","+"SD MSI events"+","+"Avg Depression episodes"+","+"SD Depression episodes"
											+","+"Avg strokes"+","+"SD strokes"+","+"Avg CHD"+","+"SD CHD"
											+","+"Avg T2D"+","+"SD T2D"+","+"Avg interventions introduced"+","+"SD interventions introduced"+","+"Avg interventions uptake"
											+","+"SD interventions uptake"+","+"Avg years active"+","+"SD years active"+","+"Avg cost"+","+"SD cost"+","+"Avg cost discounted"
											+","+"SD cost discounted"+","+"Avg event cost"+","+"SD event cost"+","+"Avg event cost discounted"+","+"SD event cost discounted"
											+","+"Avg intervention cost"+","+"SD intervention cost"+","+"Avg intervention cost discounted"+","+"SD intervention cost discounted"
											+","+"Avg QALYs"+","+"SD QALYs"+","+"Avg QALYs discounted"+","+"SD QALYs discounted"+","+"Avg intervention cost (health)"
											+","+"SD intervention cost (health)"+","+"Avg intervention cost (health) discounted"+","+"SD intervention cost (health) discounted"
											+","+"Avg intervention cost (sport)"+","+"SD intervention cost (sport)"+","+"Avg intervention cost (sport) discounted"
											+","+"SD intervention cost (sport) discounted"+","+"Avg intervention cost (education)"+","+"SD intervention cost (education)"
											+","+"Avg intervention cost (education) discounted"+","+"SD intervention cost (education) discounted"+","+"Avg intervention cost (transport)"
											+","+"SD intervention cost (transport)"+","+"Avg intervention cost (transport) discounted"+","+"SD intervention cost (transport) discounted"
											+","+"Avg intervention cost (local authorities)"+","+"SD intervention cost (local authorities)"
											+","+"Avg intervention cost (local authorities) discounted"+","+"SD intervention cost (local authorities) discounted"	
											+","+"Avg intervention cost (other)"+","+"SD intervention cost (other)"+","+"Avg intervention cost (other) discounted"
											+","+"SD intervention cost (other) discounted"
											+","+"Global number of interventions"+","+"Total fixed interventions cost"+","+"Fixed interventions cost (health)"
											+","+"Fixed interventions cost (sport)"+","+"Fixed interventions cost (education)"+","+"Fixed interventions cost (transport)"
											+","+"Fixed interventions cost (local authorities)"+","+"Fixed interventions cost (other sector)\r\n");
									bw.close();
									}
									
								}
						} catch (IOException e) {
							// Auto-generated catch block
							e.printStackTrace();
						}
					}
				//}	
				}
			}
		} 
		
		public static void writeOutIndividual(Agent a){
			try{
			
				String outfile = currentOutDirectory.getPath() + OutputFiles.INDIVIDUAL.toString().trim()+dateAppend+".csv";
				//String outfile = OutputFiles.INDIVIDUAL.toString().trim();
				FileWriter fw = new FileWriter(outfile, true);
				  BufferedWriter out = new BufferedWriter(fw);
				  out.write(PAContext.repCounter+","+a.getInModelID()+","+a.getID()+","
						  +formatNumber(a.getAgeAtEntry())+","
						  +formatNumber(a.getAgeAtSimEnd())+","
						  +formatNumber(a.getAgeAtDeath())+","
						  +a.getCauseOfDeath()+","
						  +formatNumber(a.getLifeYears())+","
						  +formatNumber(a.getPAStatusAtEntry())+","
						  +formatNumber(a.getPAStatusAtSimEnd())+","
						  +formatNumber(a.getPAStatusAtDeath())+","
						  +formatNumber(a.getBMIAtEntry())+","
						  +formatNumber(a.getBMIAtEnd())+","
						  +formatNumber(a.getHDLRatioAtEntry())+","
						  +formatNumber(a.getHDLRatioAtEnd())+","
						  +formatNumber(a.getSBPAtEntry())+","
						  +formatNumber(a.getSBPAtEnd())+","
						  +formatNumber(a.getUtiiltyAtEntry())+","
						  +formatNumber(a.getUtiiltyAtEnd())+","
						  +formatNumber(a.getYearsActive())+","
						  +a.getTotalNoOfEvents()+","
						  +a.getTotalNoOfCVD()+","
						  +a.getNoOfMSI()+","
						  +a.getNoOfDprsEpisodes()+","
						  +a.getNoOfStroke()+","
						  +a.getNoOfCHD()+","
						  +a.getNoOfT2D()+","
						  +a.getHadT2D()+","
						  +a.getHadT1D()+","
						  +a.getHadCVD()+","
						  +a.getHadMSI()+","
						  +a.getHadDepression()+","
						  +a.getSocioEconomicClass()+","
						  +a.getNameOfInterventionsIntro()+","
						  +a.getNoOfInterventionsIntro()+","
						  +a.getNameOfInterventionsUptake()+","
						  +a.getNoOfInterventionsUptake()+","
						  +formatNumber(a.getTotalQALYs())+","
						  +formatNumber(a.getTotalQALYsDiscounted())+","
						  +formatNumber(a.getTotalCost())+","
						  +formatNumber(a.getTotalCostDiscounted())+","
						  +formatNumber(a.getTotalEventCost())+","
						  +formatNumber(a.getTotalEventCostDiscounted())+","
						  +formatNumber(a.getTotalInterventionCost())+","
						  +formatNumber(a.getTotalInterventionCostDiscounted())+","
						  +formatNumber(a.getTotalInterventionCostHealth())+","
						  +formatNumber(a.getTotalInterventionCostHealthDiscounted())+","
						  +formatNumber(a.getTotalInterventionCostSport())+","
						  +formatNumber(a.getTotalInterventionCostSportDiscounted())+","
						  +formatNumber(a.getTotalInterventionCostEducation())+","
						  +formatNumber(a.getTotalInterventionCostEducationDiscounted())+","
						  +formatNumber(a.getTotalInterventionCostTransport())+","
						  +formatNumber(a.getTotalInterventionCostTransportDiscounted())+","
						  +formatNumber(a.getTotalInterventionCostLocalAuthorities())+","
						  +formatNumber(a.getTotalInterventionCostLocalAuthoritiesDiscounted())+","
						  +formatNumber(a.getTotalInterventionCostOther())+","
						  +formatNumber(a.getTotalInterventionCostOtherDiscounted())+"\r\n");
				  //Close the output stream
				  out.close();
				
			  }catch (Exception e){//Catch exception if any
				  System.err.println("Error: " + e.getMessage());
				  }
		}
		
		// write out stats for PSA and Strategy (both have the same format)
		//statsList
		
		public static void writeOut(ArrayList<Stats> statsList, String filename){
			try{
				
				FileWriter fw = new FileWriter(filename, true);
				  BufferedWriter out = new BufferedWriter(fw);
				  	  
				  out.write(PAContext.repCounter+","
						  +PAContext.cohortSize+",");
				  int idx = 0;  
					while (idx < statsList.size()){	
						out.write(
						formatNumber(statsList.get(idx).getAverage())+"," 
						  +formatNumber(statsList.get(idx++).getStDev())+",");
					}
					out.write(interventionCount+","+totalIntrventionsFixedCost+","+totalHealthFixedCost
							+","+totalSportFixedCost+","+totalEducationFixedCost+","+totalTransportFixedCost
							+","+totalLocalAuthFixedCost+","+totalOtherFixedCost+"\r\n");
				  					
				  //Close the output stream
				  out.close();
				
			  }catch (Exception e){//Catch exception if any
				  System.err.println("Error: " + e.getMessage());
				  }
		}
		
		//write out annual PSA outputs
		public static void writeOutAnnual(ArrayList<AnnualPSAOutput> annualPSAOutput, String filename){
			try{
				
				FileWriter fw = new FileWriter(filename, true);
				  BufferedWriter out = new BufferedWriter(fw);
				  
				  int idx = 0;  
					while (idx < annualPSAOutput.size()){	
						out.write(
						formatNumber(annualPSAOutput.get(idx).getYear())+","+
						formatNumber(annualPSAOutput.get(idx).getPSA())+","+
						formatNumber(annualPSAOutput.get(idx).getCohortSize())+","+
						formatNumber(annualPSAOutput.get(idx).getLifeYears().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getLifeYears().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getLifeYears().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getPAStatus().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getPAStatus().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getPAStatus().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getBMI().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getBMI().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getBMI().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getHDL().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getHDL().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getHDL().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getSBP().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getSBP().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getSBP().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getUtility().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getUtility().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getUtility().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getEvents().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getEvents().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getEvents().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getCVDEvents().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getCVDEvents().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getCVDEvents().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getMSIEvents().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getMSIEvents().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getMSIEvents().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getDepressionEpisodes().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getDepressionEpisodes().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getDepressionEpisodes().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getStrokeEvents().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getStrokeEvents().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getStrokeEvents().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getCHDEvents().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getCHDEvents().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getCHDEvents().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getT2DEvents().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getT2DEvents().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getT2DEvents().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getinterventionsIntroduced().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getinterventionsIntroduced().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getinterventionsIntroduced().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getinterventionsUptake().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getinterventionsUptake().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getinterventionsUptake().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getYearsActive().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getYearsActive().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getYearsActive().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getCost().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getCost().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getCost().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getEventCost().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getEventCost().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getEventCost().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCost().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCost().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCost().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getQALYs().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getQALYs().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getQALYs().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostHealth().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostHealth().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostHealth().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostSport().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostSport().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostSport().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostEducation().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostEducation().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostEducation().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostTransport().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostTransport().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostTransport().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostLocalAuthorities().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostLocalAuthorities().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostLocalAuthorities().getStDev())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostOther().getTotal())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostOther().getAverage())+","+
						formatNumber(annualPSAOutput.get(idx).getInterventionCostOther().getStDev())+"\r\n");
						idx++;
					}
				  //Close the output stream
				  out.close();
				
			  }catch (Exception e){//Catch exception if any
				  System.err.println("Error: " + e.getMessage());
			  }
			
			annualPSAOutput.clear();
		}
		
		// For multidimensional (2D) arrays in keeping statistics
		public static class My2DArray<T> extends ArrayList<ArrayList<T>>{
				
			
			
			////Add to arrays
			public void addToOuterArray(int index){
				while (index >=this.size()){
					this.add(new ArrayList<T>());
				}
			}
						
			public void addToInnerArray(int index, int index1, T element){
				while (index >=this.size()){
					this.add(new ArrayList<T>());
				}
				ArrayList<T> inner1 = this.get(index);
				while (index1 >= inner1.size()) {
					inner1.add(null);
				}
				inner1.set(index1, element); 
			}
						
		}
		
		
		// For multidimensional (4D) arrays in risk coefficients
		public static class My4DArray<T> extends ArrayList<ArrayList<ArrayList<ArrayList<T>>>>{
		
			
			
			////Add to arrays
			public void addToRiskArray(int index){
				while (index >=this.size()){
					this.add(new ArrayList<ArrayList<ArrayList<T>>>());
				}
			}
			
						
			public void addToMatrixArray(int index, int index1){
				while (index >=this.size()){
					this.add(new ArrayList<ArrayList<ArrayList<T>>>());
				}
				ArrayList<ArrayList<ArrayList<T>>> inner1 = this.get(index);
			    while (index1 >= inner1.size()) {
			        inner1.add(new ArrayList<ArrayList<T>>());
			    }
			}
			
			public void addToVarArray(int index, int index1, int index2){
				while (index >=this.size()){
					this.add(new ArrayList<ArrayList<ArrayList<T>>>());
				}
				ArrayList<ArrayList<ArrayList<T>>> inner1 = this.get(index);
			    while (index1 >= inner1.size()) {
			        inner1.add(new ArrayList<ArrayList<T>>());
			    }
			    ArrayList<ArrayList<T>> inner2 = inner1.get(index1);
			    while (index2 >= inner2.size()) {
			        inner2.add(new ArrayList<T>());
			    }
			}
			
			public void addToReplicationArray(int index, int index1, int index2, int index3, T element){
				while (index >=this.size()){
					this.add(new ArrayList<ArrayList<ArrayList<T>>>());
				}
				ArrayList<ArrayList<ArrayList<T>>> inner1 = this.get(index);
			    while (index1 >= inner1.size()) {
			        inner1.add(new ArrayList<ArrayList<T>>());
			    }
			    ArrayList<ArrayList<T>> inner2 = inner1.get(index1);
			    while (index2 >= inner2.size()) {
			        inner2.add(new ArrayList<T>());
			    }
			    
			    ArrayList<T> inner3 = inner2.get(index2);
			    while(index3 >= inner3.size()){
			    	inner3.add(null);
			   }
			   inner3.set(index3, element); 
			}
			
			
			
			////////get elements for QRISK CVD
			public double getQSURV_M(int variable, int repReadData){
				//variable=0 only one value (baseline) in QSURV matrix
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=0; // 0 for array index for QSURV matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQSURV_F(int variable, int repReadData){
				//variable=0 only one value (baseline) in QSURV matrix
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=0; // 0 for array index for QSURV matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQXBAR_M(int variable, int repReadData){
				//variables: seven variables in QXBAR matrix
				/*indices
				 * 0: age1
				 * 1: age2
				 * 2: bmi1
				 * 3: bmi2
				 * 4: rati
				 * 5: sbp
				 * 6: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=1; // 1 for array index for QXBAR matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQXBAR_F(int variable, int repReadData){
				//variables: seven variables in QXBAR matrix
				/*indices
				 * 0: age1
				 * 1: age2
				 * 2: bmi1
				 * 3: bmi2
				 * 4: rati
				 * 5: sbp
				 * 6: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=1; // 1 for array index for QXBAR matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQBETAeth_M(int variable, int repReadData){
				//variables: ten variables in QBETAeth matrix
				/*indices
				 * 0: White
				 * 1: Not Recorded
				 * 2: Indian
				 * 3: Pakistani
				 * 4: Bangladeshi
				 * 5: Other Asian
				 * 6: Black Carribean
				 * 7: Black African
				 * 8: Chinese
				 * 9: Other
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=2; // 2 for array index for QBETAeth matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQBETAeth_F(int variable, int repReadData){
				//variables: ten variables in QBETAeth matrix
				/*indices
				 * 0: White
				 * 1: Not Recorded
				 * 2: Indian
				 * 3: Pakistani
				 * 4: Bangladeshi
				 * 5: Other Asian
				 * 6: Black Carribean
				 * 7: Black African
				 * 8: Chinese
				 * 9: Other
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=2; // 2 for array index for QBETAeth matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQBETAsmo_M(int variable, int repReadData){
				//variables: five variables in QBETAsmo matrix
				/*indices
				 * 0: non smoker
				 * 1: ex smoker
				 * 2: light smoker
				 * 3: moderate smoker
				 * 4: heavy smoker
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=3; // 3 for array index for QBETAsmo matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQBETAsmo_F(int variable, int repReadData){
				//variables: five variables in QBETAsmo matrix
				/*indices
				 * 0: non smoker
				 * 1: ex smoker
				 * 2: light smoker
				 * 3: moderate smoker
				 * 4: heavy smoker
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=3; // 3 for array index for QBETAsmo matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQBETAcv_M(int variable, int repReadData){
				//variables: seven variables in QBETAcv matrix
				/*indices
				 * 0: age1
				 * 1: age2
				 * 2: bmi1
				 * 3: bmi2
				 * 4: rati
				 * 5: sbp
				 * 6: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=4; // 4 for array index for QBETAcv matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQBETAcv_F(int variable, int repReadData){
				//variables: seven variables in QBETAcv matrix
				/*indices
				 * 0: age1
				 * 1: age2
				 * 2: bmi1
				 * 3: bmi2
				 * 4: rati
				 * 5: sbp
				 * 6: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=4; // 4 for array index for QBETAcv matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQBETAbv_M(int variable, int repReadData){
				//variables: seven variables in QBETAbv matrix
				/*indices
				 * 0: af
				 * 1: ra
				 * 2: ckd
				 * 3: aht
				 * 4: dm1
				 * 5: dm2
				 * 6: FHcvd
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=5; // 5 for array index for QBETAbv matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQBETAbv_F(int variable, int repReadData){
				//variables: seven variables in QBETAbv matrix
				/*indices
				 * 0: af
				 * 1: ra
				 * 2: ckd
				 * 3: aht
				 * 4: dm1
				 * 5: dm2
				 * 6: FHcvd
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=5; // 5 for array index for QBETAbv matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQAGE1SMO_M(int variable, int repReadData){
				//variables: five variables in QAGE1SMO matrix
				/*indices
				 * 0: non smoker
				 * 1: ex smoker
				 * 2: light smoker
				 * 3: moderate smoker
				 * 4: heavy smoker
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=6; // 6 for array index for QAGE1SMO matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQAGE1SMO_F(int variable, int repReadData){
				//variables: five variables in QAGE1SMO matrix
				/*indices
				 * 0: non smoker
				 * 1: ex smoker
				 * 2: light smoker
				 * 3: moderate smoker
				 * 4: heavy smoker
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=6; // 6 for array index for QAGE1SMO matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQAGE1INT_M(int variable, int repReadData){
				//variables: ten variables in QAGE1INT matrix
				/*indices
				 * 0: af
				 * 1: ckd
				 * 2: aht
				 * 3: dm1
				 * 4: dm2
				 * 5: bmi1
				 * 6: bmi2
				 * 7: FHcvd
				 * 8: sbp
				 * 9: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=7; // 7 for array index for QAGE1INT matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQAGE1INT_F(int variable, int repReadData){
				//variables: ten variables in QAGE1INT matrix
				/*indices
				 * 0: af
				 * 1: ckd
				 * 2: aht
				 * 3: dm1
				 * 4: dm2
				 * 5: bmi1
				 * 6: bmi2
				 * 7: FHcvd
				 * 8: sbp
				 * 9: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=7; // 7 for array index for QAGE1INT matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQAGE2SMO_M(int variable, int repReadData){
				//variables: five variables in QAGE2SMO matrix
				/*indices
				 * 0: non smoker
				 * 1: ex smoker
				 * 2: light smoker
				 * 3: moderate smoker
				 * 4: heavy smoker
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=8; // 8 for array index for QAGE2SMO matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQAGE2SMO_F(int variable, int repReadData){
				//variables: five variables in QAGE2SMO matrix
				/*indices
				 * 0: non smoker
				 * 1: ex smoker
				 * 2: light smoker
				 * 3: moderate smoker
				 * 4: heavy smoker
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=8; // 8 for array index for QAGE2SMO matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			
			public double getQAGE2INT_M(int variable, int repReadData){
				//variables: ten variables in QAGE2INT matrix
				/*indices
				 * 0: af
				 * 1: ckd
				 * 2: aht
				 * 3: dm1
				 * 4: dm2
				 * 5: bmi1
				 * 6: bmi2
				 * 7: FHcvd
				 * 8: sbp
				 * 9: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=0; // 0 for array index for Male risk
				int matrix=9; // 9 for array index for QAGE9INT matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
			public double getQAGE2INT_F(int variable, int repReadData){
				//variables: ten variables in QAGE2INT matrix
				/*indices
				 * 0: af
				 * 1: ckd
				 * 2: aht
				 * 3: dm1
				 * 4: dm2
				 * 5: bmi1
				 * 6: bmi2
				 * 7: FHcvd
				 * 8: sbp
				 * 9: town
				*/
				//repReadData = replication -1 --> set in Agent class for all PSA readings
				int risk=1; // 1 for array index for Female risk
				int matrix=9; // 9 for array index for QAGE2INT matrix
				
				double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
				
				return myValue;
			}
		
			////////get elements for QRISK DIABETES
				public double getDSURV_M(int variable, int repReadData){
					//variable=0 only one value (baseline) in DSURV matrix
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=0; // 0 for array index for DSURV matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDSURV_F(int variable, int repReadData){
					//variable=0 only one value (baseline) in DSURV matrix
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=0; // 0 for array index for DSURV matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDXBAR_M(int variable, int repReadData){
					//variables: five variables in DXBAR matrix
					/*indices
					 * 0: age1
					 * 1: age2
					 * 2: bmi1
					 * 3: bmi2
					 * 4: town
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=1; // 1 for array index for DXBAR matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDXBAR_F(int variable, int repReadData){
					//variables: five variables in DXBAR matrix
					/*indices
					 * 0: age1
					 * 1: age2
					 * 2: bmi1
					 * 3: bmi2
					 * 4: town
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=1; // 1 for array index for DXBAR matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDBETAeth_M(int variable, int repReadData){
					//variables: ten variables in DBETAeth matrix
					/*indices
					 * 0: White
					 * 1: Not Recorded
					 * 2: Indian
					 * 3: Pakistani
					 * 4: Bangladeshi
					 * 5: Other Asian
					 * 6: Black Carribean
					 * 7: Black African
					 * 8: Chinese
					 * 9: Other
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=2; // 2 for array index for DBETAeth matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDBETAeth_F(int variable, int repReadData){
					//variables: ten variables in DBETAeth matrix
					/*indices
					 * 0: White
					 * 1: Not Recorded
					 * 2: Indian
					 * 3: Pakistani
					 * 4: Bangladeshi
					 * 5: Other Asian
					 * 6: Black Carribean
					 * 7: Black African
					 * 8: Chinese
					 * 9: Other
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=2; // 2 for array index for DBETAeth matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDBETAsmo_M(int variable, int repReadData){
					//variables: five variables in DBETAsmo matrix
					/*indices
					 * 0: non smoker
					 * 1: ex smoker
					 * 2: light smoker
					 * 3: moderate smoker
					 * 4: heavy smoker
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=3; // 3 for array index for DBETAsmo matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDBETAsmo_F(int variable, int repReadData){
					//variables: five variables in DBETAsmo matrix
					/*indices
					 * 0: non smoker
					 * 1: ex smoker
					 * 2: light smoker
					 * 3: moderate smoker
					 * 4: heavy smoker
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=3; // 3 for array index for DBETAsmo matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDBETAcv_M(int variable, int repReadData){
					//variables: five variables in DBETAcv matrix
					/*indices
					 * 0: age1
					 * 1: age2
					 * 2: bmi1
					 * 3: bmi2
					 * 4: town
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=4; // 4 for array index for DBETAcv matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDBETAcv_F(int variable, int repReadData){
					//variables: five variables in DBETAcv matrix
					/*indices
					 * 0: age1
					 * 1: age2
					 * 2: bmi1
					 * 3: bmi2
					 * 4: town
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=4; // 4 for array index for DBETAcv matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDBETAbv_M(int variable, int repReadData){
					//variables: four variables in DBETAbv matrix
					/*indices
					 * 0: cort
					 * 1: cvd
					 * 2: aht
					 * 3: FHdm2
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=5; // 5 for array index for DBETAbv matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDBETAbv_F(int variable, int repReadData){
					//variables: four variables in DBETAbv matrix
					/*indices
					 * 0: cort
					 * 1: cvd
					 * 2: aht
					 * 3: FHdm2
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=5; // 5 for array index for DBETAbv matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDAGE1INT_M(int variable, int repReadData){
					//variables: three variables in DAGE1INT matrix
					/*indices
					 * 0: bmi1
					 * 1: bmi2
					 * 2: FHdm2
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=6; // 6 for array index for DAGE1INT matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDAGE1INT_F(int variable, int repReadData){
					//variables: three variables in DAGE1INT matrix
					/*indices
					 * 0: bmi1
					 * 1: bmi2
					 * 2: FHdm2
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=6; // 6 for array index for DAGE1INT matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				
				public double getDAGE2INT_M(int variable, int repReadData){
					//variables: three variables in DAGE2INT matrix
					/*indices
					 * 0: bmi1
					 * 1: bmi2
					 * 2: FHdm2
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=0; // 0 for array index for Male risk
					int matrix=7; // 7 for array index for DAGE9INT matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
				public double getDAGE2INT_F(int variable, int repReadData){
					//variables: three variables in DAGE2INT matrix
					/*indices
					 * 0: bmi1
					 * 1: bmi2
					 * 2: FHdm2
					*/
					//repReadData = replication -1 --> set in Agent class for all PSA readings
					int risk=1; // 1 for array index for Female risk
					int matrix=7; // 7 for array index for DAGE2INT matrix
					
					double myValue = Double.valueOf(this.get(risk).get(matrix).get(variable).get(repReadData).toString().trim());
					
					return myValue;
				}
		}
		
		public static double calculateQRisk(Agent a, int repReadData){
			double qRisk, sum;
			
			//variables for individual agent 
			int sex, eth, smo, af, ra, ckd, aht, dm1, dm2, fhcvd; 
			double age, bmi, rati, sbp, town;
			if (a.getGender().equalsIgnoreCase("M")){sex = 0;} else {sex = 1;}
			age = a.getAge();
			bmi = a.getBMI();
			rati = a.getHDLRatio();
			sbp = a.getSBloodPressure();
			town = a.getDeprivationTown();
			if(a.getEthnicity().equalsIgnoreCase("White")){eth = 0;}
			else if (a.getEthnicity().equalsIgnoreCase("NR")){eth = 1;}
			else if (a.getEthnicity().equalsIgnoreCase("Indian")){eth = 2;}
			else if (a.getEthnicity().equalsIgnoreCase("Pakistani")){eth = 3;}
			else if (a.getEthnicity().equalsIgnoreCase("Bangladeshi")){eth = 4;}
			else if (a.getEthnicity().equalsIgnoreCase("Other_Asian")){eth = 5;}
			else if (a.getEthnicity().equalsIgnoreCase("Black_Carribean")){eth = 6;}
			else if (a.getEthnicity().equalsIgnoreCase("Black_African")){eth = 7;}
			else if (a.getEthnicity().equalsIgnoreCase("Chinese")){eth = 8;}
			else {eth = 9;}
			smo = a.getSmoking(); //0: non, 1: ex, 2: light, 3: moderate, 4: heavy
			af = a.getAtrialFibrillation();
			ra = a.getRheumatoidArthritis();
			ckd = a.getChronicKidneyDisease();
			aht = a.getBpTreatment();
			dm1 = a.getHadT1D();
			if(a.hasT2D()){dm2 = 1;}
			else{dm2 =0;}
			
			fhcvd = a.getFamilyCVDHistory();
			
			//variables for QRisk calculation
			double dage, age1, age2, dbmi, bmi1, bmi2; 
						
			//fit
			if (bmi > 40){bmi = 40;} else if (bmi < 20) {bmi = 20;}
			if (sbp > 210) {sbp = 210;} else if (sbp < 70) {sbp = 70;}
				
			dage = age/10;
			// MALE: sex = 0; FEMALE: sex = 1
			if (sex==1) {age1 = Math.pow(dage, 0.5); age2 = dage;} else {age1 = Math.pow(dage, -1); age2 = Math.pow(dage, 2);}
						
			dbmi = bmi/10;
			bmi1 = Math.pow(dbmi, -2);
			bmi2 = bmi1*Math.log(dbmi);
						
			if (sex == 0) { //MALE
				//Centre
				
				age1 = age1 - PAContext.qRiskCoef.getQXBAR_M(0, repReadData);
				age2 = age2 - PAContext.qRiskCoef.getQXBAR_M(1, repReadData);
				
				bmi1 = bmi1 - PAContext.qRiskCoef.getQXBAR_M(2, repReadData);
				bmi2 = bmi2 - PAContext.qRiskCoef.getQXBAR_M(3, repReadData);
				rati = rati - PAContext.qRiskCoef.getQXBAR_M(4, repReadData);
				sbp = sbp - PAContext.qRiskCoef.getQXBAR_M(5, repReadData);
				town = town - PAContext.qRiskCoef.getQXBAR_M(6, repReadData);
				
				//Sums
				sum = 0;
				
				sum = PAContext.qRiskCoef.getQBETAeth_M(eth, repReadData) + PAContext.qRiskCoef.getQBETAsmo_M(smo, repReadData);
					
				sum = sum + (age1*PAContext.qRiskCoef.getQBETAcv_M(0, repReadData)) + (age2*PAContext.qRiskCoef.getQBETAcv_M(1, repReadData)) 
						+ (bmi1*PAContext.qRiskCoef.getQBETAcv_M(2, repReadData)) + (bmi2*PAContext.qRiskCoef.getQBETAcv_M(3, repReadData))
						+ (rati*PAContext.qRiskCoef.getQBETAcv_M(4, repReadData)) + (sbp*PAContext.qRiskCoef.getQBETAcv_M(5, repReadData))
						+ (town*PAContext.qRiskCoef.getQBETAcv_M(6, repReadData));
		
				sum = sum + (af*PAContext.qRiskCoef.getQBETAbv_M(0, repReadData)) + (ra*PAContext.qRiskCoef.getQBETAbv_M(1, repReadData))
						+ (ckd*PAContext.qRiskCoef.getQBETAbv_M(2, repReadData)) + (aht*PAContext.qRiskCoef.getQBETAbv_M(3, repReadData))
						+ (dm1*PAContext.qRiskCoef.getQBETAbv_M(4, repReadData)) + (dm2*PAContext.qRiskCoef.getQBETAbv_M(5, repReadData))
						+ (fhcvd*PAContext.qRiskCoef.getQBETAbv_M(6, repReadData));
				
				sum = sum + (age1*PAContext.qRiskCoef.getQAGE1SMO_M(smo, repReadData));
				
				sum = sum + (age1*af*PAContext.qRiskCoef.getQAGE1INT_M(0, repReadData)) + (age1*ckd*PAContext.qRiskCoef.getQAGE1INT_M(1, repReadData))
						+ (age1*aht*PAContext.qRiskCoef.getQAGE1INT_M(2, repReadData)) + (age1*dm1*PAContext.qRiskCoef.getQAGE1INT_M(3, repReadData))
						+ (age1*dm2*PAContext.qRiskCoef.getQAGE1INT_M(4, repReadData)) + (age1*bmi1*PAContext.qRiskCoef.getQAGE1INT_M(5, repReadData))
						+ (age1*bmi2*PAContext.qRiskCoef.getQAGE1INT_M(6, repReadData)) + (age1*fhcvd*PAContext.qRiskCoef.getQAGE1INT_M(7, repReadData))
						+ (age1*sbp*PAContext.qRiskCoef.getQAGE1INT_M(8, repReadData)) + (age1*town*PAContext.qRiskCoef.getQAGE1INT_M(9, repReadData));
				
				sum = sum + (age2*PAContext.qRiskCoef.getQAGE2SMO_M(smo, repReadData));
				
				sum = sum + (age2*af*PAContext.qRiskCoef.getQAGE2INT_M(0, repReadData)) + (age2*ckd*PAContext.qRiskCoef.getQAGE2INT_M(1, repReadData))
						+ (age2*aht*PAContext.qRiskCoef.getQAGE2INT_M(2, repReadData)) + (age2*dm1*PAContext.qRiskCoef.getQAGE2INT_M(3, repReadData))
						+ (age2*dm2*PAContext.qRiskCoef.getQAGE2INT_M(4, repReadData)) + (age2*bmi1*PAContext.qRiskCoef.getQAGE2INT_M(5, repReadData))
						+ (age2*bmi2*PAContext.qRiskCoef.getQAGE2INT_M(6, repReadData)) + (age2*fhcvd*PAContext.qRiskCoef.getQAGE2INT_M(7, repReadData))
						+ (age2*sbp*PAContext.qRiskCoef.getQAGE2INT_M(8, repReadData)) + (age2*town*PAContext.qRiskCoef.getQAGE2INT_M(9, repReadData));
											
				qRisk = 1 - Math.pow(PAContext.qRiskCoef.getQSURV_M(0, repReadData), Math.exp(sum)); 
			
			}
			else { //FEMALE
				//Centre
				age1 = age1 - PAContext.qRiskCoef.getQXBAR_F(0, repReadData);
				age2 = age2 - PAContext.qRiskCoef.getQXBAR_F(1, repReadData);
				
				bmi1 = bmi1 - PAContext.qRiskCoef.getQXBAR_F(2, repReadData);
				bmi2 = bmi2 - PAContext.qRiskCoef.getQXBAR_F(3, repReadData);
				rati = rati - PAContext.qRiskCoef.getQXBAR_F(4, repReadData);
				sbp = sbp - PAContext.qRiskCoef.getQXBAR_F(5, repReadData);
				town = town - PAContext.qRiskCoef.getQXBAR_F(6, repReadData);
				
				//Sums
				sum = 0;
				
				sum = PAContext.qRiskCoef.getQBETAeth_F(eth, repReadData) + PAContext.qRiskCoef.getQBETAsmo_F(smo, repReadData);
				
				sum = sum + (age1*PAContext.qRiskCoef.getQBETAcv_F(0, repReadData)) + (age2*PAContext.qRiskCoef.getQBETAcv_F(1, repReadData)) 
						+ (bmi1*PAContext.qRiskCoef.getQBETAcv_F(2, repReadData)) + (bmi2*PAContext.qRiskCoef.getQBETAcv_F(3, repReadData))
						+ (rati*PAContext.qRiskCoef.getQBETAcv_F(4, repReadData)) + (sbp*PAContext.qRiskCoef.getQBETAcv_F(5, repReadData))
						+ (town*PAContext.qRiskCoef.getQBETAcv_F(6, repReadData));
				
				sum = sum + (af*PAContext.qRiskCoef.getQBETAbv_F(0, repReadData)) + (ra*PAContext.qRiskCoef.getQBETAbv_F(1, repReadData))
						+ (ckd*PAContext.qRiskCoef.getQBETAbv_F(2, repReadData)) + (aht*PAContext.qRiskCoef.getQBETAbv_F(3, repReadData))
						+ (dm1*PAContext.qRiskCoef.getQBETAbv_F(4, repReadData)) + (dm2*PAContext.qRiskCoef.getQBETAbv_F(5, repReadData))
						+ (fhcvd*PAContext.qRiskCoef.getQBETAbv_F(6, repReadData));
				
				sum = sum + (age1*PAContext.qRiskCoef.getQAGE1SMO_F(smo, repReadData));
				
				sum = sum + (age1*af*PAContext.qRiskCoef.getQAGE1INT_F(0, repReadData)) + (age1*ckd*PAContext.qRiskCoef.getQAGE1INT_F(1, repReadData))
						+ (age1*aht*PAContext.qRiskCoef.getQAGE1INT_F(2, repReadData)) + (age1*dm1*PAContext.qRiskCoef.getQAGE1INT_F(3, repReadData))
						+ (age1*dm2*PAContext.qRiskCoef.getQAGE1INT_F(4, repReadData)) + (age1*bmi1*PAContext.qRiskCoef.getQAGE1INT_F(5, repReadData))
						+ (age1*bmi2*PAContext.qRiskCoef.getQAGE1INT_F(6, repReadData)) + (age1*fhcvd*PAContext.qRiskCoef.getQAGE1INT_F(7, repReadData))
						+ (age1*sbp*PAContext.qRiskCoef.getQAGE1INT_F(8, repReadData)) + (age1*town*PAContext.qRiskCoef.getQAGE1INT_F(9, repReadData));
				
				sum = sum + (age2*PAContext.qRiskCoef.getQAGE2SMO_F(smo, repReadData));
				
				sum = sum + (age2*af*PAContext.qRiskCoef.getQAGE2INT_F(0, repReadData)) + (age2*ckd*PAContext.qRiskCoef.getQAGE2INT_F(1, repReadData))
						+ (age2*aht*PAContext.qRiskCoef.getQAGE2INT_F(2, repReadData)) + (age2*dm1*PAContext.qRiskCoef.getQAGE2INT_F(3, repReadData))
						+ (age2*dm2*PAContext.qRiskCoef.getQAGE2INT_F(4, repReadData)) + (age2*bmi1*PAContext.qRiskCoef.getQAGE2INT_F(5, repReadData))
						+ (age2*bmi2*PAContext.qRiskCoef.getQAGE2INT_F(6, repReadData)) + (age2*fhcvd*PAContext.qRiskCoef.getQAGE2INT_F(7, repReadData))
						+ (age2*sbp*PAContext.qRiskCoef.getQAGE2INT_F(8, repReadData)) + (age2*town*PAContext.qRiskCoef.getQAGE2INT_F(9, repReadData));
				
				qRisk = 1 - Math.pow(PAContext.qRiskCoef.getQSURV_F(0, repReadData), Math.exp(sum)); 
			}
			
				
			return qRisk;
		} 
		
		public static double calculateDRisk(Agent a, int repReadData){
					
			double dRisk, sum;
			
			//variables for individual agent 
			int sex, eth, smo, cort, cvd, aht, fhdm2; 
			double age, bmi, town;
			
			if (a.getGender().equalsIgnoreCase("M")){sex = 0;} else {sex = 1;}
			age = a.getAge();
			bmi = a.getBMI();
			
			town = a.getDeprivationTown();
			if(a.getEthnicity().equalsIgnoreCase("White")){eth = 0;}
			else if (a.getEthnicity().equalsIgnoreCase("NR")){eth = 1;}
			else if (a.getEthnicity().equalsIgnoreCase("Indian")){eth = 2;}
			else if (a.getEthnicity().equalsIgnoreCase("Pakistani")){eth = 3;}
			else if (a.getEthnicity().equalsIgnoreCase("Bangladeshi")){eth = 4;}
			else if (a.getEthnicity().equalsIgnoreCase("Other_Asian")){eth = 5;}
			else if (a.getEthnicity().equalsIgnoreCase("Black_Carribean")){eth = 6;}
			else if (a.getEthnicity().equalsIgnoreCase("Black_African")){eth = 7;}
			else if (a.getEthnicity().equalsIgnoreCase("Chinese")){eth = 8;}
			else {eth = 9;}
			smo = a.getSmoking(); //0: non, 1: ex, 2: light, 3: moderate, 4: heavy
			aht = a.getBpTreatment();
			fhdm2 = a.getFamilyT2DHistory();
			cort = a.getCort();
			if (a.hasCVD()) {cvd = 1;} else {cvd = 0;}
			
			//variables for QRisk calculation
			double dage, age1, age2, dbmi, bmi1, bmi2; 
						
			//fit
			if (bmi > 40){bmi = 40;} else if (bmi < 20) {bmi = 20;}
				
			//TODO check with Catriona and Nana
			dage = age/10;
			if (sex == 1) {age1 = Math.pow(dage, 0.5); age2 = Math.pow(dage, 3);} else {age1 = Math.log(dage); age2 = Math.pow(dage, 3);}
						
			dbmi = bmi/10;
			if (sex == 1) {bmi1 = dbmi; bmi2 = Math.pow(dbmi, 3);} else {bmi1 = Math.pow(dbmi, 2); bmi2 = Math.pow(dbmi, 3);}
						
			if (sex == 0) { //MALE
				//Centre
				age1 = age1 - PAContext.dRiskCoef.getDXBAR_M(0, repReadData);
				age2 = age2 - PAContext.dRiskCoef.getDXBAR_M(1, repReadData);
				
				bmi1 = bmi1 - PAContext.dRiskCoef.getDXBAR_M(2, repReadData);
				bmi2 = bmi2 - PAContext.dRiskCoef.getDXBAR_M(3, repReadData);
				town = town - PAContext.dRiskCoef.getDXBAR_M(4, repReadData);
				
				//Sums
				sum = 0;
				
				sum = PAContext.dRiskCoef.getDBETAeth_M(eth, repReadData) + PAContext.dRiskCoef.getDBETAsmo_M(smo, repReadData);
				
				sum = sum + (age1*PAContext.dRiskCoef.getDBETAcv_M(0, repReadData)) + (age2*PAContext.dRiskCoef.getDBETAcv_M(1, repReadData)) 
						+ (bmi1*PAContext.dRiskCoef.getDBETAcv_M(2, repReadData)) + (bmi2*PAContext.dRiskCoef.getDBETAcv_M(3, repReadData))
						+ (town*PAContext.dRiskCoef.getDBETAcv_M(4, repReadData));
				
				sum = sum + (cort*PAContext.dRiskCoef.getDBETAbv_M(0, repReadData)) + (cvd*PAContext.dRiskCoef.getDBETAbv_M(1, repReadData))
						+ (aht*PAContext.dRiskCoef.getDBETAbv_M(2, repReadData)) + (fhdm2*PAContext.dRiskCoef.getDBETAbv_M(3, repReadData));
				
				sum = sum + (age1*bmi1*PAContext.dRiskCoef.getDAGE1INT_M(0, repReadData)) + (age1*bmi2*PAContext.dRiskCoef.getDAGE1INT_M(1, repReadData))
						+ (age1*fhdm2*PAContext.dRiskCoef.getDAGE1INT_M(2, repReadData));
						
				sum = sum + (age2*bmi1*PAContext.dRiskCoef.getDAGE2INT_M(0, repReadData)) + (age2*bmi2*PAContext.dRiskCoef.getDAGE2INT_M(1, repReadData))
						+ (age2*fhdm2*PAContext.dRiskCoef.getDAGE2INT_M(2, repReadData));
				
				dRisk = 1 - Math.pow(PAContext.dRiskCoef.getDSURV_M(0, repReadData), Math.exp(sum)); 
			}
			else { //FEMALE
				//Centre
				age1 = age1 - PAContext.dRiskCoef.getDXBAR_F(0, repReadData);
				age2 = age2 - PAContext.dRiskCoef.getDXBAR_F(1, repReadData);
				
				bmi1 = bmi1 - PAContext.dRiskCoef.getDXBAR_F(2, repReadData);
				bmi2 = bmi2 - PAContext.dRiskCoef.getDXBAR_F(3, repReadData);
				town = town - PAContext.dRiskCoef.getDXBAR_F(4, repReadData);
				
				//Sums
				sum = 0;
				
				sum = PAContext.dRiskCoef.getDBETAeth_F(eth, repReadData) + PAContext.dRiskCoef.getDBETAsmo_F(smo, repReadData);
				
				sum = sum + (age1*PAContext.dRiskCoef.getDBETAcv_F(0, repReadData)) + (age2*PAContext.dRiskCoef.getDBETAcv_F(1, repReadData)) 
						+ (bmi1*PAContext.dRiskCoef.getDBETAcv_F(2, repReadData)) + (bmi2*PAContext.dRiskCoef.getDBETAcv_F(3, repReadData))
						+ (town*PAContext.dRiskCoef.getDBETAcv_F(4, repReadData));
				
				sum = sum + (cort*PAContext.dRiskCoef.getDBETAbv_F(0, repReadData)) + (cvd*PAContext.dRiskCoef.getDBETAbv_F(1, repReadData))
						+ (aht*PAContext.dRiskCoef.getDBETAbv_F(2, repReadData)) + (fhdm2*PAContext.dRiskCoef.getDBETAbv_F(3, repReadData));
				
				sum = sum + (age1*bmi1*PAContext.dRiskCoef.getDAGE1INT_F(0, repReadData)) + (age1*bmi2*PAContext.dRiskCoef.getDAGE1INT_F(1, repReadData))
						+ (age1*fhdm2*PAContext.dRiskCoef.getDAGE1INT_F(2, repReadData));
						
				sum = sum + (age2*bmi1*PAContext.dRiskCoef.getDAGE2INT_F(0, repReadData)) + (age2*bmi2*PAContext.dRiskCoef.getDAGE2INT_F(1, repReadData))
						+ (age2*fhdm2*PAContext.dRiskCoef.getDAGE2INT_F(2, repReadData));
				
				dRisk = 1 - Math.pow(PAContext.dRiskCoef.getDSURV_F(0, repReadData), Math.exp(sum)); 
			}
			
				
			return dRisk;
		} 
		
		
		
		
		public static double calculateTimeAdjustedProbability(Agent a, double probability, double timeCurrent, double timeNew){
			double rate, returnProb;	
			
			rate = (-1*(Math.log(1-probability)))/timeCurrent;
			returnProb = 1-Math.exp((-1*rate)*timeNew);
			return returnProb;			
		}
		
		public static double rrPAEventRisk(Agent a, My2DArray<Double> paOn){
			double rrRisk, rr = 0;
			double paStatus = a.getPAStatus();		
			int pa = 100; 
			
			if (paStatus >= 100*paOn.size()){rr = (paOn.get(paOn.size()-1).get(a.repReadData));}
			else{
				for(int i = 0; i < paOn.size(); i++){
					if (paStatus<pa){rr = paOn.get(i).get(a.repReadData);
						break;
					}
					pa+=100;
				}
			}
			
			rrRisk = rr;
			return rrRisk;
		}
		
		public static double rrPADprsProbability(Agent a){
			double rrDepProbability;
			double paStatus = a.getPAStatus();
		
			if(paStatus <= 69){rrDepProbability = PAContext.rrPAOnDepression.get(0).get(a.repReadData);}
			else if(paStatus>=70 && paStatus<=209){rrDepProbability = PAContext.rrPAOnDepression.get(1).get(a.repReadData);}
			else if(paStatus>=210 && paStatus<=419){rrDepProbability = PAContext.rrPAOnDepression.get(2).get(a.repReadData);}
			else if(paStatus>=420 && paStatus<=629){rrDepProbability = PAContext.rrPAOnDepression.get(3).get(a.repReadData);}
			else{rrDepProbability = PAContext.rrPAOnDepression.get(4).get(a.repReadData);}
			
			return rrDepProbability;
		}
	
		public static double rrPAmsiProbability(Agent a){
			double rrMSIProbability;
			double paStatus = a.getPAStatus();
			
			if(a.getGender().equalsIgnoreCase("M")){
				if(paStatus <= 59){rrMSIProbability = PAContext.rrPAOnMSI.get(0).get(a.repReadData);}
				else if(paStatus>=60 && paStatus<=300){rrMSIProbability = PAContext.rrPAOnMSI.get(1).get(a.repReadData);}
				else if(paStatus>=301 && paStatus<=600){rrMSIProbability = PAContext.rrPAOnMSI.get(2).get(a.repReadData);}
				else{rrMSIProbability = PAContext.rrPAOnMSI.get(3).get(a.repReadData);}
			}
			else{
				if(paStatus <= 59){rrMSIProbability = PAContext.rrPAOnMSI.get(4).get(a.repReadData);}
				else if(paStatus>=60 && paStatus<=300){rrMSIProbability = PAContext.rrPAOnMSI.get(5).get(a.repReadData);}
				else if(paStatus>=301 && paStatus<=600){rrMSIProbability = PAContext.rrPAOnMSI.get(6).get(a.repReadData);}
				else{rrMSIProbability = PAContext.rrPAOnMSI.get(7).get(a.repReadData);}
			}
			
			return rrMSIProbability;
		}
		
		
		public static double cvdStrokeProportion(Agent a){
			double cvdStrokeProportion;
			double age = a.getAge();
		/*
			int aa = 35; 
			
			for(int i = 0; i < PAContext.proportionStrokeCHD.size()/2; i++){
				if(age < aa){
					cvdStrokeProportion = PAContext.proportionStrokeCHD.get(i).get(a.repReadData);
					break;
				}
				aa+=10;
			}
			*/
			
				if(age < 35){cvdStrokeProportion = PAContext.proportionStrokeCHD.get(0).get(a.repReadData);}
				else if(age >=35 && age <= 44){cvdStrokeProportion = PAContext.proportionStrokeCHD.get(1).get(a.repReadData);}
				else if(age >= 45 && age <= 54){cvdStrokeProportion = PAContext.proportionStrokeCHD.get(2).get(a.repReadData);}
				else if(age >= 55 && age <= 64){cvdStrokeProportion = PAContext.proportionStrokeCHD.get(3).get(a.repReadData);}
				else if(age >= 65 && age <= 74){cvdStrokeProportion = PAContext.proportionStrokeCHD.get(4).get(a.repReadData);}
				else if(age >= 75 && age <= 84){cvdStrokeProportion = PAContext.proportionStrokeCHD.get(5).get(a.repReadData);}
				else{cvdStrokeProportion = PAContext.proportionStrokeCHD.get(6).get(a.repReadData);}
					
			return cvdStrokeProportion;
		}
		
		public static double cvdCHDProportion(Agent a){
			double cvdCHDProportion;
			double age = a.getAge();
		
				if(age <= 35){cvdCHDProportion = PAContext.proportionStrokeCHD.get(7).get(a.repReadData);}
				else if(age >=35 && age <= 44){cvdCHDProportion = PAContext.proportionStrokeCHD.get(8).get(a.repReadData);}
				else if(age >= 45 && age <= 54){cvdCHDProportion = PAContext.proportionStrokeCHD.get(9).get(a.repReadData);}
				else if(age >= 55 && age <= 64){cvdCHDProportion = PAContext.proportionStrokeCHD.get(10).get(a.repReadData);}
				else if(age >= 65 && age <= 74){cvdCHDProportion = PAContext.proportionStrokeCHD.get(11).get(a.repReadData);}
				else if(age >= 75 && age <= 84){cvdCHDProportion = PAContext.proportionStrokeCHD.get(12).get(a.repReadData);}
				else{cvdCHDProportion = PAContext.proportionStrokeCHD.get(13).get(a.repReadData);}
					
			return cvdCHDProportion;
		}
		
		public static double fatalityCHD(Agent a){
			double fatalityCHD;
			double age = a.getAge();
			
			if(age < 30){fatalityCHD = PAContext.fatalityCHD.get(0).get(a.repReadData);}
			else if(age >=30 && age <= 54){fatalityCHD = PAContext.fatalityCHD.get(1).get(a.repReadData);}
			else if(age >= 55 && age <= 64){fatalityCHD = PAContext.fatalityCHD.get(2).get(a.repReadData);}
			else if(age >= 65 && age <= 74){fatalityCHD = PAContext.fatalityCHD.get(3).get(a.repReadData);}
			else if(age >= 75 && age <= 84){fatalityCHD = PAContext.fatalityCHD.get(4).get(a.repReadData);}
			else{fatalityCHD = PAContext.fatalityCHD.get(5).get(a.repReadData);}
					
			return fatalityCHD;
		}
		
		public static double fatalityStroke(Agent a){
			double fatalityStroke;
			double age = a.getAge();
			
			if(a.getGender().equalsIgnoreCase("M")){
				if(age<75){fatalityStroke = PAContext.fatalityStroke.get(0).get(a.repReadData);}
				else{fatalityStroke = PAContext.fatalityStroke.get(1).get(a.repReadData);}
			}
			else{
				if(age<75){fatalityStroke = PAContext.fatalityStroke.get(2).get(a.repReadData);}
				else{fatalityStroke = PAContext.fatalityStroke.get(3).get(a.repReadData);}
			}
					
			return fatalityStroke;
		}
		
		
		//TODO check with Nana discounted values method
		//Method to calculate discount for a simulation time unit based on daily increment and daily discount rate
		public static double calculateDiscount(double dailyIncrement, double dailyDiscountRate){
			double discounted = 0;
			int dt = (int) Util.CONVERT_DAYS;
			discounted = (dailyIncrement * (1 - Math.pow(1+dailyDiscountRate, dt))) / dailyDiscountRate;
			return discounted;
		}
		
		
		public static void updateAgeRelatedCharacteristics(Agent a){
			
			double bmi, hdl, sbp;
			double ageSq3Mnths = a.getAge()*(12/3);
			
			if(a.getGender().equalsIgnoreCase("M")){
				bmi = a.getBMI() + 0.124 - 0.00027*ageSq3Mnths;
				hdl = a.getHDLRatio() + 0.0207 - 0.000045*ageSq3Mnths;
				sbp = a.getSBloodPressure() + 0.179 - 0.00023*ageSq3Mnths;
				
			}
			else{
				bmi = a.getBMI() + 0.107 - 0.00023*ageSq3Mnths;
				hdl = a.getHDLRatio() + 0.0134 - 0.000027*ageSq3Mnths;
				sbp = a.getSBloodPressure() + 0.1 - 0.000027*ageSq3Mnths;
			}
			
			a.setBMI(bmi);
			a.setHDLRatio(hdl);
			a.setSBloodPressure(sbp);
		}
		
		
		public static void getEvent(ArrayList<OccuredEvent> historyList, Record rec, Agent a){
			OccuredEvent oe = new OccuredEvent(rec, a);
			historyList.add(oe);
			//System.out.println("in get event");
			
			a.setTotalNoOfEvents(a.getTotalNoOfEvents()+1);
			
			if (oe.getEventName().equalsIgnoreCase(Events.DEPRESSION.toString().trim())){
				a.setNoOfDprsEpisodes(a.getNoOfDprsEpisodes()+1);
			}
			else if (oe.getEventName().equalsIgnoreCase(Events.MSI.toString().trim())){
				a.setNoOfMSI(a.getNoOfMSI()+1);
			}
			else if (oe.getEventName().equalsIgnoreCase(Events.STROKE.toString().trim())){
				a.setNoOfStroke(a.getNoOfStroke()+1);
				a.setTotalNoOfCVD(a.getTotalNoOfCVD()+1);
			}
			else if (oe.getEventName().equalsIgnoreCase(Events.CHD.toString().trim())){
				a.setNoOfCHD(a.getNoOfCHD()+1);
				a.setTotalNoOfCVD(a.getTotalNoOfCVD()+1);
			}
			else if (oe.getEventName().equalsIgnoreCase(Events.T2D.toString().trim())){
				a.setNoOfT2D(a.getNoOfT2D()+1);
			}
		}
		
		public static void introduceIntervention(ArrayList<IntroducedIntervention> historyList, Record rec, Agent a){
			IntroducedIntervention introIn = new IntroducedIntervention(rec, a);
			introIn.setIntroducedIntervention(true);
			historyList.add(introIn);
			
			a.setNoOfInterventionsIntro(a.getNoOfInterventionsIntro()+1);
			a.setNameOfInterventionsIntro(rec.getInterventionName(a.repReadData));
		}
		
		public static void uptakeIntervention(ArrayList<UptakeIntervention> historyList, Record rec, Agent a){
			UptakeIntervention upIn = new UptakeIntervention(rec, a);
			upIn.setIntroducedIntervention(true);
			upIn.setUptookIntervention(true);
			historyList.add(upIn);
			
			a.setNoOfInterventionsUptake(a.getNoOfInterventionsUptake()+1);
			a.setNameOfInterventionsUptake(rec.getInterventionName(a.repReadData));
		}
		
		
				
		 
		public static void updateBaselinePAStatus (Agent a){
			double pa, meanAge, sdAge, ndAge, meanAgeSq, sdAgeSq, ndAgeSq;
			double ageSq3Mnths = a.getAge()*(12/3);
			/*
			System.out.println("");
			System.out.println("PA before baseline PA update");
			System.out.println("pa previous: "+a.getPAPreviousPeriod());
			System.out.println("pa current: "+a.getPAStatus());
			System.out.println("END ## PA before baseline PA update");
			System.out.println("");
			//*/
			if(a.getGender().equalsIgnoreCase("M")){
				if(a.getPAStatus() < PA_MODERATELY_THRESHOLD){
					meanAge = 0.002; sdAge = 1.248; ndAge = RandomHelper.createNormal(meanAge, sdAge).nextDouble();
					meanAgeSq = 0.0001; sdAgeSq = 0; ndAgeSq = RandomHelper.createNormal(meanAgeSq, sdAgeSq).nextDouble();
					pa = a.getPAStatus() + ndAge - ndAgeSq*ageSq3Mnths;
					if(pa<0){pa=0;}
					if(pa>=3600){pa=3600;} // 8.5 hours per day -- max in database
					a.setPAStatus(pa);
					
				}
				else if(a.getPAStatus() >= PA_MODERATELY_THRESHOLD && a.getPAStatus() < PA_VERY_THRESHOLD){
					meanAge = 0.122; sdAge = 0.015; ndAge = RandomHelper.createNormal(meanAge, sdAge).nextDouble();
					meanAgeSq = 0.0001; sdAgeSq = 0; ndAgeSq = RandomHelper.createNormal(meanAgeSq, sdAgeSq).nextDouble();
					pa = a.getPAStatus() - ndAge + ndAgeSq*ageSq3Mnths;
					if(pa<0){pa=0;}
					if(pa>=3600){pa=3600;} // 8.5 hours per day -- max in database
					a.setPAStatus(pa);
					
				}
				else {
					meanAge = 3.865; sdAge = 34.256; ndAge = RandomHelper.createNormal(meanAge, sdAge).nextDouble();
					meanAgeSq = 0.010; sdAgeSq = 0.121; ndAgeSq = RandomHelper.createNormal(meanAgeSq, sdAgeSq).nextDouble();
					pa = a.getPAStatus() + ndAge - ndAgeSq*ageSq3Mnths;
					if(pa<0){pa=0;}
					if(pa>=3600){pa=3600;} // 8.5 hours per day -- max in database
					a.setPAStatus(pa);
				
				}
			}
			else{
				if(a.getPAStatus() < PA_MODERATELY_THRESHOLD){
					meanAge = 0.040; sdAge = 1.276; ndAge = RandomHelper.createNormal(meanAge, sdAge).nextDouble();
					meanAgeSq = 0.0002; sdAgeSq = 0; ndAgeSq = RandomHelper.createNormal(meanAgeSq, sdAgeSq).nextDouble();
					pa = a.getPAStatus() + ndAge - ndAgeSq*ageSq3Mnths;
					if(pa<0){pa=0;}
					if(pa>=3600){pa=3600;} // 8.5 hours per day -- max in database
					a.setPAStatus(pa);
				}
				else if(a.getPAStatus() >= PA_MODERATELY_THRESHOLD && a.getPAStatus() < PA_VERY_THRESHOLD){
					meanAge = 0.116; sdAge = 4.873; ndAge = RandomHelper.createNormal(meanAge, sdAge).nextDouble();
					meanAgeSq = 0.0001; sdAgeSq = 0; ndAgeSq = RandomHelper.createNormal(meanAgeSq, sdAgeSq).nextDouble();
					pa = a.getPAStatus() - ndAge + ndAgeSq*ageSq3Mnths;
					if(pa<0){pa=0;}
					if(pa>=3600){pa=3600;} // 8.5 hours per day -- max in database
					a.setPAStatus(pa);
				}
				else {
					meanAge = 4.328; sdAge = 30.490; ndAge = RandomHelper.createNormal(meanAge, sdAge).nextDouble();
					meanAgeSq = 0.012; sdAgeSq = 0.079; ndAgeSq = RandomHelper.createNormal(meanAgeSq, sdAgeSq).nextDouble();
					pa = a.getPAStatus() + ndAge - ndAgeSq*ageSq3Mnths;
					if(pa<0){pa=0;}
					if(pa>=3600){pa=3600;} // 8.5 hours per day -- max in database
					a.setPAStatus(pa);
				}
			}
			/*
			System.out.println("distributiuon values every baseline PA update");
			System.out.println("mean age "+meanAge);
			System.out.println("sd age "+ sdAge);
			System.out.println("nd value age "+ndAge);
			System.out.println("mean age sq "+meanAgeSq);
			System.out.println("sd age sq "+sdAgeSq);
			System.out.println("nd value age sq"+ndAgeSq);
			System.out.println("pa: "+pa);
			System.out.println("END ## distributiuon values every baseline PA update");
			System.out.println("");
			//*/
		}
		
		
		public static void updateBaselineUtility(Agent a){
			
			double util;
			double ageSq3Mnths = a.getAge()*(12/3);
			
			if(a.getGender().equalsIgnoreCase("M")){
				util = a.getBaselineUtility() - 0.00000841 - 0.000000484*ageSq3Mnths;
				}
			else{
				util = a.getBaselineUtility() - 0.0000666 - 0.00000029*ageSq3Mnths; 
			}
			
			if(util>1){util=1;}
			
			a.setBaselineUtility(util);
			a.setCurrentUtility(a.getBaselineUtility());
		}
		
		public static void eventUpdates(Agent a){
			
			double t = getCurrentTime();
		
			if (a.getCHDEventsList().size()!=0){
			//	System.out.println("in CHD list");
				if(getMonths(t-a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getStartTime()) < a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventAcuteDuration()){
					//System.out.println("in CHD acute");
					a.updateTotalEventCost(a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventAcuteUtilityMultiplier()*a.getBaselineUtility());
					a.setPAStatus(a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventAcutePA());
					a.setInAcutePeriod(true);
				}
				else{
					//System.out.println("in CHD post acute");
					a.updateTotalEventCost(a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventPostAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventPostAcuteUtilityMultiplier()*a.getBaselineUtility());
				}
				
				if(getMonths(t-a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getStartTime()) == a.getCHDEventsList().get(a.getCHDEventsList().size()-1).getEventAcuteDuration()){
					a.getCHDEventsList().get(a.getCHDEventsList().size()-1).setPostPAStatus(a);
					//System.out.println("in CHD post acute PA");
					//System.out.println(a.getPAStatus());
					a.setInAcutePeriod(false);
				}
			
			}
			if (a.getStrokeEventsList().size()!=0){
				//System.out.println("in Stroke list");
				if(getMonths(t-a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getStartTime()) < a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getEventAcuteDuration()){
					//System.out.println("in Stroke acute");
					a.updateTotalEventCost(a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getEventAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getEventAcuteUtilityMultiplier()*a.getBaselineUtility());
					a.setInAcutePeriod(true);
				}
				else{
					//System.out.println("in Stroke post acute");
					a.updateTotalEventCost(a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getEventPostAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getEventPostAcuteUtilityMultiplier()*a.getBaselineUtility());
				}
				
				if(getMonths(t-a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getStartTime()) == a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).getEventAcuteDuration()){
					a.getStrokeEventsList().get(a.getStrokeEventsList().size()-1).setPostPAStatus(a);
					//System.out.println("in Stroke post acute PA");
					//System.out.println(a.getPAStatus());
					a.setInAcutePeriod(false);
				}
			}
			if (a.getT2DEventsList().size()!=0){
				//System.out.println("in T2D list");
				if(getMonths(t-a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getStartTime()) < a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getEventAcuteDuration()){
					//System.out.println("in T2D acute");
					a.updateTotalEventCost(a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getEventAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getEventAcuteUtilityMultiplier()*a.getBaselineUtility());
					//a.setInAcutePeriod(true);
				}
				else{
					//System.out.println("in T2D post acute");
					a.updateTotalEventCost(a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getEventPostAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getEventPostAcuteUtilityMultiplier()*a.getBaselineUtility());
				}
				
				if(getMonths(t-a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getStartTime()) == a.getT2DEventsList().get(a.getT2DEventsList().size()-1).getEventAcuteDuration()){
					a.getT2DEventsList().get(a.getT2DEventsList().size()-1).setPostPAStatus(a);
					//System.out.println("in T2D post acute PA");
					//System.out.println(a.getPAStatus());
					a.setInAcutePeriod(false);
				}
			}
			if (a.getMSIEventsList().size()!=0){
				//System.out.println("in MSI list");
				//System.out.println(a.getMSIEventsList().size());
				if(getMonths(t-a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getStartTime()) < a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getEventAcuteDuration()){
					//System.out.println("in MSI acute");
					a.updateTotalEventCost(a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getEventAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getEventAcuteUtilityMultiplier()*a.getBaselineUtility());
					a.setInAcutePeriod(true);
				}
				else{
					//System.out.println("in MSI post acute");
					a.updateTotalEventCost(a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getEventPostAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getEventPostAcuteUtilityMultiplier()*a.getBaselineUtility());
				}
				
				if(getMonths(t-a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getStartTime()) == a.getMSIEventsList().get(a.getMSIEventsList().size()-1).getEventAcuteDuration()){
					a.getMSIEventsList().get(a.getMSIEventsList().size()-1).setPostPAStatus(a);
					//System.out.println("in MSI post acute PA");
					//System.out.println(a.getPAStatus());
					a.setInAcutePeriod(false);
				}
			}
			if (a.getDepressionEventsList().size()!=0){
				//System.out.println("in Depression list");
				if(getMonths(t-a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getStartTime()) < a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getEventAcuteDuration()){
				//	System.out.println("in Depression acute");
					a.updateTotalEventCost(a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getEventAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getEventAcuteUtilityMultiplier()*a.getBaselineUtility());
					a.setInAcutePeriod(true);
				}
				else{
					//System.out.println("in Depression post acute");
					a.updateTotalEventCost(a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getEventPostAcuteCost()*CONVERT_MONTHS);
					a.setCurrentUtility(a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getEventPostAcuteUtilityMultiplier()*a.getBaselineUtility());
				}
				
				if(getMonths(t-a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getStartTime()) == a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).getEventAcuteDuration()){
					a.getDepressionEventsList().get(a.getDepressionEventsList().size()-1).setPostPAStatus(a);
				//	System.out.println("in Depression post acute PA");
				//	System.out.println(a.getPAStatus());
					a.setInAcutePeriod(false);
				}
			}
			
		}
		
		public static void interventionUpdates(Agent a){
			
			double t = getCurrentTime();
			
			if (a.getUptakeIntervention().size()!=0){
				
				for (Iterator i = a.getUptakeIntervention().iterator();i.hasNext();){
					UptakeIntervention upInt = (UptakeIntervention) i.next();
					if(getMonths(t-upInt.getStartTime()) == upInt.getEffectStartTimeMonths()){
						double mean = upInt.getEffectOnPAMean();
						double sd = upInt.getEffectOnPASD();
						double paChange = RandomHelper.createNormal(mean, sd).nextDouble();
						a.setPAStatus(a.getPAStatus()+paChange);
						if(a.getPAStatus()<0){a.setPAStatus(0);}
					}
					
				if(getMonths(t-upInt.getStartTime()) < upInt.getInterventionDuration()){
						
						a.setTotalInterventionCostHealth(a.getTotalInterventionCostHealth()+upInt.getHealthIndividualCostMonth()*CONVERT_MONTHS);
						a.setTotalInterventionCostSport(a.getTotalInterventionCostSport()+upInt.getSportIndividualCostMonth()*CONVERT_MONTHS);
						a.setTotalInterventionCostEducation(a.getTotalInterventionCostEducation()+upInt.getEducationIndividualCostMonth()*CONVERT_MONTHS);
						a.setTotalInterventionCostTransport(a.getTotalInterventionCostTransport()+upInt.getTransportIndividualCostMonth()*CONVERT_MONTHS);
						a.setTotalInterventionCostLocalAuthorities(a.getTotalInterventionCostLocalAuthorities()+upInt.getLocalAuthoritiesIndividualCostMonth()*CONVERT_MONTHS);
						a.setTotalInterventionCostOther(a.getTotalInterventionCostOther()+upInt.getOtherIndividualCostMonth()*CONVERT_MONTHS);
						
						a.updateTotalInterventionCost(a.getTotalInterventionCostHealth()+a.getTotalInterventionCostSport()
								+a.getTotalInterventionCostEducation()+a.getTotalInterventionCostTransport()
								+a.getTotalInterventionCostLocalAuthorities()+a.getTotalInterventionCostOther());
						
					}
				}
			} 
		}
		
		//@ScheduledMethod(start=4, interval=4, priority=ScheduleParameters.LAST_PRIORITY-1)
		public static void recordAnnualPSAOutput(){
			
			AnnualPSAOutput year = new AnnualPSAOutput();
			Context context = RunState.getInstance().getMasterContext();
			for (Object obj : context) {
				if(obj instanceof Agent){
					
					year.getLifeYears().recordDT(((Agent) obj).getLifeYears());
					year.getPAStatus().recordDT(((Agent) obj).getPAStatus());
					year.getBMI().recordDT(((Agent) obj).getBMI());
					year.getHDL().recordDT(((Agent) obj).getHDLRatio());
					year.getSBP().recordDT(((Agent) obj).getSBloodPressure());
					year.getUtility().recordDT(((Agent) obj).getBaselineUtility());
					year.getEvents().recordDT(((Agent) obj).getTotalNoOfEvents());
					year.getCVDEvents().recordDT(((Agent) obj).getTotalNoOfCVD()); 
					year.getMSIEvents().recordDT(((Agent) obj).getNoOfMSI());
					year.getDepressionEpisodes().recordDT(((Agent) obj).getNoOfDprsEpisodes());
					year.getStrokeEvents().recordDT(((Agent) obj).getNoOfStroke());
					year.getCHDEvents().recordDT(((Agent) obj).getNoOfCHD());
					year.getT2DEvents().recordDT(((Agent) obj).getNoOfT2D());
					year.getinterventionsIntroduced().recordDT(((Agent) obj).getNoOfInterventionsIntro());
					year.getinterventionsUptake().recordDT(((Agent) obj).getNoOfInterventionsUptake());
					year.getYearsActive().recordDT(((Agent) obj).getYearsActive());
					year.getCost().recordDT(((Agent) obj).getTotalCost());
					year.getEventCost().recordDT(((Agent) obj).getTotalEventCost());
					year.getInterventionCost().recordDT(((Agent) obj).getTotalInterventionCost());
					year.getQALYs().recordDT(((Agent) obj).getTotalQALYs());
					year.getInterventionCostHealth().recordDT(((Agent) obj).getTotalInterventionCostHealth());
					year.getInterventionCostSport().recordDT(((Agent) obj).getTotalInterventionCostSport());
					year.getInterventionCostEducation().recordDT(((Agent) obj).getTotalInterventionCostEducation());
					year.getInterventionCostTransport().recordDT(((Agent) obj).getTotalInterventionCostTransport());
					year.getInterventionCostLocalAuthorities().recordDT(((Agent) obj).getTotalInterventionCostLocalAuthorities());
					year.getInterventionCostOther().recordDT(((Agent) obj).getTotalInterventionCostOther());
				}
			}
			PAContext.psaAnnualStatsList.add(year);	
		}
		
}
